@@ -1,4 +1,4 @@
-/* $Id: color.c,v 1.7 2004/07/31 14:10:23 dolorous Exp $ */
+/* $Id: color.c,v 1.9 2004/11/17 23:17:05 jordi Exp $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
@@ -19,13 +19,14 @@
  *                                                                        *
  **************************************************************************/
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 #include "proto.h"
 #include "nano.h"
