@@ -1,4 +1,4 @@
-/* $Id: cut.c,v 1.9 2004/08/12 03:27:54 dolorous Exp $ */
+/* $Id: cut.c,v 1.20 2004/11/17 23:17:05 jordi Exp $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -19,7 +19,9 @@
  *                                                                        *
  **************************************************************************/
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <string.h>
@@ -30,11 +32,10 @@
 
 static bool keep_cutbuffer = FALSE;
 	/* Should we keep the contents of the cutbuffer? */
-static int marked_cut;
-	/* Is the cutbuffer from a mark?  0 means whole-line cut, 1
-	 * means mark, and 2 means cut-from-cursor. */
+static cut_type marked_cut = CUT_LINE;
+	/* What type of cut is in the cutbuffer? */
 #ifndef NANO_SMALL
-static bool concatenate_cut;
+static bool concatenate_cut = FALSE;
 	/* Should we add this cut string to the end of the last one? */
 #endif
 static filestruct *cutbottom = NULL;
@@ -60,8 +61,8 @@ void add_to_cutbuffer(filestruct *inptr, bool allow_concat)
 	cutbuffer = inptr;
 #ifndef NANO_SMALL
     else if (allow_concat && concatenate_cut) {
-	/* Just tack the text in inptr onto the text in cutbottom,
-	 * unless allow_concat is FALSE. */
+	/* If allow_concat is TRUE and we're concatenating, tack the
+	 * text in inptr onto the text in cutbottom. */
 	cutbottom->data = charealloc(cutbottom->data,
 		strlen(cutbottom->data) + strlen(inptr->data) + 1);
 	strcat(cutbottom->data, inptr->data);
@@ -106,7 +107,7 @@ void cut_marked_segment(void)
     /* Set up the top and bottom lines and coordinates of the marked
      * text. */
     mark_order((const filestruct **)&top, &top_x,
-		(const filestruct **)&bot, &bot_x);
+		(const filestruct **)&bot, &bot_x, NULL);
 
     /* Make the first cut line manually.  Move the cut part of the top
      * line into tmp, and set newsize to that partial line's length. */
@@ -121,9 +122,26 @@ void cut_marked_segment(void)
 	cutbuffer = tmp;
 	cutbottom = tmp;
     } else {
+	if (concatenate_cut) {
+	    /* If we're concatenating, tack the text in the first line
+	     * of tmp onto the text in the bottom of the cutbuffer, and
+	     * move tmp one line down to where its next line begins. */
+	    cutbottom->data = charealloc(cutbottom->data,
+		strlen(cutbottom->data) + strlen(tmp->data) + 1);
+	    strcat(cutbottom->data, tmp->data);
+	    tmp = tmp->next;
+	}
+
+	/* Put tmp on the line after the bottom of the cutbuffer. */
 	cutbottom->next = tmp;
-	tmp->prev = cutbottom;
-	cutbottom = tmp;
+
+	if (!concatenate_cut) {
+	    /* Tf we're not concatenating, attach tmp to the bottom of
+	     * the cutbuffer, and then move the bottom of the cutbuffer
+	     * one line down to where tmp is. */
+	    tmp->prev = cutbottom;
+	    cutbottom = tmp;
+	}
     }
 
     /* And make the top remainder line manually too.  Update current_x
@@ -205,12 +223,12 @@ void do_cut_text(void)
 
     assert(current != NULL && current->data != NULL);
 
-    check_statblank();
+    check_statusblank();
 
     if (!keep_cutbuffer) {
 	free_filestruct(cutbuffer);
 	cutbuffer = NULL;
-	marked_cut = 0;
+	marked_cut = CUT_LINE;
 #ifndef NANO_SMALL
 	concatenate_cut = FALSE;
 #endif
@@ -223,26 +241,25 @@ void do_cut_text(void)
      * does clear the cutbuffer if keep_cutbuffer is FALSE. */
     if (current == filebot
 #ifndef NANO_SMALL
-			&& !ISSET(MARK_ISSET)
+		&& !ISSET(MARK_ISSET)
 #endif
-						)
+		)
 	return;
 
     keep_cutbuffer = TRUE;
 
 #ifndef NANO_SMALL
     if (ISSET(CUT_TO_END) && !ISSET(MARK_ISSET)) {
-	assert(current_x >= 0 && current_x <= strlen(current->data));
+	assert(current_x <= strlen(current->data));
 
 	if (current->data[current_x] == '\0') {
 	    /* If the line is empty and we didn't just cut a non-blank
 	     * line, create a dummy blank line and add it to the
 	     * cutbuffer. */
-	    if (marked_cut != 1 && current->next != filebot) {
+	    if (marked_cut != CUT_MARKED && current->next != filebot) {
 		filestruct *junk = make_new_node(current);
 
-		junk->data = charalloc(1);
-		junk->data[0] = '\0';
+		junk->data = mallocstrcpy(NULL, "");
 		add_to_cutbuffer(junk, TRUE);
 #ifdef DEBUG
 		dump_buffer(cutbuffer);
@@ -250,7 +267,7 @@ void do_cut_text(void)
 	    }
 
 	    do_delete();
-	    marked_cut = 2;
+	    marked_cut = CUT_TO_EOL;
 	    return;
 	} else {
 	    SET(MARK_ISSET);
@@ -266,12 +283,9 @@ void do_cut_text(void)
 	placewewant = xplustabs();
 	UNSET(MARK_ISSET);
 
-	/* If we just did a marked cut of part of a line, we should add
-	 * the first line of any cut done immediately afterward to the
-	 * end of this cut, as Pico does. */
-	if (current == mark_beginbuf && current_x < strlen(current->data))
-	    concatenate_cut = TRUE;
-	marked_cut = 1;
+	marked_cut = CUT_MARKED;
+	concatenate_cut = TRUE;
+
 	edit_refresh();
 	set_modified();
 	return;
@@ -300,7 +314,7 @@ void do_cut_text(void)
     current_x = 0;
     edit_refresh();
     set_modified();
-    marked_cut = 0;
+    marked_cut = CUT_LINE;
 #ifndef NANO_SMALL
     concatenate_cut = FALSE;
 #endif
@@ -315,7 +329,7 @@ void do_uncut_text(void)
 #ifndef DISABLE_WRAPPING
     wrap_reset();
 #endif
-    check_statblank();
+    check_statusblank();
     if (cutbuffer == NULL || current == NULL)
 	return;			/* AIEEEEEEEEEEEE */
 
@@ -323,9 +337,9 @@ void do_uncut_text(void)
      * we're not at the beginning of the line.  If we are at the
      * beginning of the line, set placewewant to 0.  Pico does both of
      * these. */
-    if (marked_cut == 0) {
-	if (current_x != 0)
-	    marked_cut = 2;
+    if (marked_cut == CUT_LINE) {
+	if (current_x > 0)
+	    marked_cut = CUT_TO_EOL;
 	else
 	    placewewant = 0;
     }
@@ -335,7 +349,7 @@ void do_uncut_text(void)
     if (current->next == NULL)
 	new_magicline();
 
-    if (marked_cut == 0 || cutbuffer->next != NULL) {
+    if (marked_cut == CUT_LINE || cutbuffer->next != NULL) {
 	newbuf = copy_filestruct(cutbuffer);
 	for (newend = newbuf; newend->next != NULL && newend != NULL;
 		newend = newend->next)
@@ -343,7 +357,7 @@ void do_uncut_text(void)
     }
 
     /* Hook newbuf in at current. */
-    if (marked_cut != 0) {
+    if (marked_cut != CUT_LINE) {
 	filestruct *hold = current;
 
 	/* If there's only one line in the cutbuffer... */
@@ -351,10 +365,12 @@ void do_uncut_text(void)
 	    size_t buf_len = strlen(cutbuffer->data);
 	    size_t cur_len = strlen(current->data);
 
-	    current->data = charealloc(current->data, cur_len + buf_len + 1);
+	    current->data = charealloc(current->data, cur_len +
+		buf_len + 1);
 	    charmove(current->data + current_x + buf_len,
-			current->data + current_x, cur_len - current_x + 1);
-	    strncpy(current->data + current_x, cutbuffer->data, buf_len);
+		current->data + current_x, cur_len - current_x + 1);
+	    strncpy(current->data + current_x, cutbuffer->data,
+		buf_len);
 		/* Use strncpy() to not copy the null terminator. */
 
 	    current_x += buf_len;
@@ -374,7 +390,7 @@ void do_uncut_text(void)
 
 	    /* New end. */
 	    tmpstr2 = charalloc(strlen(newend->data) +
-			      strlen(&current->data[current_x]) + 1);
+		strlen(&current->data[current_x]) + 1);
 	    strcpy(tmpstr2, newend->data);
 	    strcat(tmpstr2, &current->data[current_x]);
 
@@ -401,17 +417,19 @@ void do_uncut_text(void)
 		new_magicline();
 	    }
 
-	    /* Now why don't we update the totsize also? */
-	    for (tmp = current->next; tmp != newend; tmp = tmp->next)
+	    /* Recalculate current_y and totsize. */
+	    for (tmp = current->next; tmp != newend; tmp = tmp->next) {
+		current_y++;
 		totsize += strlen(tmp->data) + 1;
+	    }
 
 	    current = newend;
 	}
 
-	/* If marked cut == 2, it means that we're doing a cut to end
-	 * and we don't want anything else on the line, so we have to
-	 * screw up all the work we just did and separate the line. */
-	if (marked_cut == 2) {
+	/* If we're doing a cut to end, we don't want anything else on
+	 * the line, so we have to screw up all the work we just did and
+	 * separate the line. */
+	if (marked_cut == CUT_TO_EOL) {
 	    tmp = make_new_node(current);
 	    tmp->data = mallocstrcpy(NULL, current->data + current_x);
 	    splice_node(current, tmp, current->next);
@@ -424,6 +442,7 @@ void do_uncut_text(void)
 	    totlines++;
 	    totsize++;
 	}
+
 	/* Renumber from BEFORE where we pasted ;) */
 	renumber(hold);
 
@@ -442,6 +461,7 @@ void do_uncut_text(void)
 	newbuf->prev = tmp;
     } else
 	fileage = newbuf;
+
     totlines++;		/* Unmarked uncuts don't split lines. */
 
     /* This is so uncutting at the top of the buffer will work => */
@@ -452,9 +472,11 @@ void do_uncut_text(void)
     newend->next = current;
     current->prev = newend;
 
-    /* Recalculate size *sigh* */
-    for (tmp = newbuf; tmp != current; tmp = tmp->next)
+    /* Recalculate current_y and totsize. */
+    for (tmp = newbuf; tmp != current; tmp = tmp->next) {
+	current_y++;
 	totsize += strlen(tmp->data) + 1;
+    }
 
     renumber(newbuf);
     edit_refresh();
