@@ -1,4 +1,4 @@
-/* $Id: global.c,v 1.49 2004/08/17 05:23:38 dolorous Exp $ */
+/* $Id: global.c,v 1.72 2004/11/17 23:17:05 jordi Exp $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -19,11 +19,12 @@
  *                                                                        *
  **************************************************************************/
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <assert.h>
-#include <sys/stat.h>
 #include "proto.h"
 #include "nano.h"
 
@@ -52,15 +53,18 @@ struct stat originalfilestat;	/* Stat for the file as we loaded it */
 int editwinrows = 0;		/* How many rows long is the edit
 				   window? */
 filestruct *current;		/* Current buffer pointer */
-int current_x = 0, current_y = 0;	/* Current position of X and Y in
-					   the editor - relative to edit
-					   window (0,0) */
+size_t current_x = 0;		/* Current x-coordinate in the edit
+				   window */
+int current_y = 0;		/* Current y-coordinate in the edit
+				   window */
 filestruct *fileage = NULL;	/* Our file buffer */
 filestruct *edittop = NULL;	/* Pointer to the top of the edit
 				   buffer with respect to the
 				   file struct */
 filestruct *filebot = NULL;	/* Last node in the file struct */
 filestruct *cutbuffer = NULL;	/* A place to store cut text */
+partition *filepart = NULL;	/* A place to store a portion of the
+				   file struct */
 
 #ifdef ENABLE_MULTIBUFFER
 openfilestruct *open_files = NULL;	/* The list of open files */
@@ -93,8 +97,6 @@ size_t quotelen;		/* strlen(quotestr) */
 char *backup_dir = NULL;	/* Backup directory. */
 #endif
 
-int resetstatuspos;		/* Hack for resetting the status bar 
-				   cursor position */
 char *answer = NULL;		/* Answer str to many questions */
 int totlines = 0;		/* Total number of lines in the file */
 long totsize = 0;		/* Total number of bytes in the file */
@@ -114,7 +116,7 @@ char *help_text;		/* The text in the help window */
 
 #ifndef NANO_SMALL
 filestruct *mark_beginbuf;	/* The begin marker buffer */
-int mark_beginx;		/* X value in the string to start */
+size_t mark_beginx;		/* X value in the string to start */
 #endif
 
 #ifndef DISABLE_OPERATINGDIR
@@ -130,7 +132,7 @@ shortcut *main_list = NULL;
 shortcut *whereis_list = NULL;
 shortcut *replace_list = NULL;
 shortcut *replace_list_2 = NULL; 	/* 2nd half of replace dialog */
-shortcut *goto_list = NULL;
+shortcut *gotoline_list = NULL;
 shortcut *writefile_list = NULL;
 shortcut *insertfile_list = NULL;
 #ifndef DISABLE_HELP
@@ -224,7 +226,7 @@ void sc_init_one(shortcut **shortcutage, int ctrlval, const char *desc,
     s->next = NULL;
 }
 
-void shortcut_init(int unjustify)
+void shortcut_init(bool unjustify)
 {
     const char *get_help_msg = N_("Get Help");
     const char *exit_msg = N_("Exit");
@@ -235,6 +237,11 @@ void shortcut_init(int unjustify)
     const char *cancel_msg = N_("Cancel");
     const char *first_line_msg = N_("First Line");
     const char *last_line_msg = N_("Last Line");
+#ifndef DISABLE_JUSTIFY
+    const char *beg_of_par_msg = N_("Beg of Par");
+    const char *end_of_par_msg = N_("End of Par");
+    const char *fulljstify_msg = N_("FullJstify");
+#endif
 #ifndef NANO_SMALL
     const char *case_sens_msg = N_("Case Sens");
     const char *direction_msg = N_("Direction");
@@ -242,6 +249,9 @@ void shortcut_init(int unjustify)
     const char *regexp_msg = N_("Regexp");
 #endif
     const char *history_msg = N_("History");
+#ifdef ENABLE_MULTIBUFFER
+    const char *new_buffer_msg = N_("New Buffer");
+#endif
 #endif /* !NANO_SMALL */
 #ifndef DISABLE_BROWSER
     const char *to_files_msg = N_("To Files");
@@ -269,8 +279,12 @@ void shortcut_init(int unjustify)
 	N_("Uncut from the cutbuffer into the current line");
     const char *nano_cursorpos_msg = N_("Show the position of the cursor");
     const char *nano_spell_msg = N_("Invoke the spell checker, if available");
-    const char *nano_goto_msg = N_("Go to a specific line number");
+    const char *nano_gotoline_msg = N_("Go to a specific line number");
     const char *nano_replace_msg = N_("Replace text within the editor");
+#ifndef NANO_SMALL
+    const char *nano_mark_msg = N_("Mark text at the cursor position");
+    const char *nano_whereis_next_msg = N_("Repeat last search");
+#endif
     const char *nano_prevline_msg = N_("Move to the previous line");
     const char *nano_nextline_msg = N_("Move to the next line");
     const char *nano_forward_msg = N_("Move forward one character");
@@ -278,38 +292,37 @@ void shortcut_init(int unjustify)
     const char *nano_home_msg = N_("Move to the beginning of the current line");
     const char *nano_end_msg = N_("Move to the end of the current line");
     const char *nano_refresh_msg = N_("Refresh (redraw) the current screen");
-    const char *nano_mark_msg = N_("Mark text at the current cursor location");
     const char *nano_delete_msg = N_("Delete the character under the cursor");
     const char *nano_backspace_msg =
 	N_("Delete the character to the left of the cursor");
-    const char *nano_tab_msg = N_("Insert a tab character");
+    const char *nano_tab_msg =
+	N_("Insert a tab character at the cursor position");
     const char *nano_enter_msg =
 	N_("Insert a carriage return at the cursor position");
 #ifndef NANO_SMALL
     const char *nano_nextword_msg = N_("Move forward one word");
     const char *nano_prevword_msg = N_("Move backward one word");
 #endif
-    const char *nano_verbatim_msg = N_("Insert character(s) verbatim");
-#ifdef ENABLE_MULTIBUFFER
-    const char *nano_openprev_msg = N_("Switch to previous file buffer");
-    const char *nano_opennext_msg = N_("Switch to next file buffer");
-#endif
-#ifndef NANO_SMALL
-#ifdef HAVE_REGEX_H
-    const char *nano_bracket_msg = N_("Find other bracket");
-#endif
-    const char *nano_whereis_next_msg = N_("Repeat last search");
-#endif
-    const char *nano_cancel_msg = N_("Cancel the current function");
-    const char *nano_firstline_msg = N_("Go to the first line of the file");
-    const char *nano_lastline_msg = N_("Go to the last line of the file");
 #ifndef DISABLE_JUSTIFY
     const char *nano_parabegin_msg =
 	N_("Go to the beginning of the current paragraph");
     const char *nano_paraend_msg =
 	N_("Go to the end of the current paragraph");
+#endif
+#ifdef ENABLE_MULTIBUFFER
+    const char *nano_openprev_msg = N_("Switch to the previous file buffer");
+    const char *nano_opennext_msg = N_("Switch to the next file buffer");
+#endif
+    const char *nano_verbatim_msg = N_("Insert character(s) verbatim");
+#ifndef DISABLE_JUSTIFY
     const char *nano_fulljustify_msg = N_("Justify the entire file");
 #endif
+#if !defined(NANO_SMALL) && defined(HAVE_REGEX_H)
+    const char *nano_bracket_msg = N_("Find other bracket");
+#endif
+    const char *nano_cancel_msg = N_("Cancel the current function");
+    const char *nano_firstline_msg = N_("Go to the first line of the file");
+    const char *nano_lastline_msg = N_("Go to the last line of the file");
 #ifndef NANO_SMALL
     const char *nano_case_msg =
 	N_("Make the current search/replace case (in)sensitive");
@@ -367,7 +380,8 @@ void shortcut_init(int unjustify)
 		);
 
 #ifdef ENABLE_MULTIBUFFER
-    if (open_files != NULL && (open_files->prev != NULL || open_files->next != NULL))
+    if (open_files != NULL && (open_files->prev != NULL ||
+	open_files->next != NULL))
     /* Translators: try to keep this string under 10 characters long */
 	sc_init_one(&main_list, NANO_EXIT_KEY, N_("Close"),
 		IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
@@ -409,7 +423,8 @@ void shortcut_init(int unjustify)
 #else
 		NOVIEW
 #endif
-		, !ISSET(RESTRICTED) ? do_insertfile_void : nano_disabled_msg);
+		, !ISSET(RESTRICTED) ? do_insertfile_void :
+		nano_disabled_msg);
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_WHEREIS_KEY, N_("Where Is"),
@@ -459,13 +474,23 @@ void shortcut_init(int unjustify)
 #endif
 		nano_disabled_msg);
 
-    sc_init_one(&main_list, NANO_GOTO_KEY, go_to_line_msg,
-	IFHELP(nano_goto_msg, NANO_ALT_GOTO_KEY), NANO_GOTO_FKEY,
-	NANO_NO_KEY, VIEW, do_gotoline_void);
+    sc_init_one(&main_list, NANO_GOTOLINE_KEY, go_to_line_msg,
+	IFHELP(nano_gotoline_msg, NANO_GOTOLINE_ALTKEY),
+	NANO_GOTOLINE_FKEY, NANO_NO_KEY, VIEW, do_gotoline_void);
 
     sc_init_one(&main_list, NANO_REPLACE_KEY, replace_msg,
-	IFHELP(nano_replace_msg, NANO_ALT_REPLACE_KEY), NANO_REPLACE_FKEY,
-	NANO_NO_KEY, NOVIEW, do_replace);
+	IFHELP(nano_replace_msg, NANO_ALT_REPLACE_KEY),
+	NANO_REPLACE_FKEY, NANO_NO_KEY, NOVIEW, do_replace);
+
+#ifndef NANO_SMALL
+    sc_init_one(&main_list, NANO_MARK_KEY, N_("Mark Text"),
+	IFHELP(nano_mark_msg, NANO_MARK_ALTKEY), NANO_MARK_FKEY,
+	NANO_NO_KEY, NOVIEW, do_mark);
+
+    sc_init_one(&main_list, NANO_NO_KEY, N_("Where Is Next"),
+	IFHELP(nano_whereis_next_msg, NANO_WHEREIS_NEXT_KEY),
+	NANO_WHEREIS_NEXT_FKEY, NANO_NO_KEY, VIEW, do_research);
+#endif
 
     sc_init_one(&main_list, NANO_PREVLINE_KEY, N_("Prev Line"),
 	IFHELP(nano_prevline_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -495,16 +520,6 @@ void shortcut_init(int unjustify)
 	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, total_refresh);
 
-    sc_init_one(&main_list, NANO_MARK_KEY, N_("Mark Text"),
-	IFHELP(nano_mark_msg, NANO_ALT_MARK_KEY),
-	NANO_NO_KEY, NANO_NO_KEY, NOVIEW,
-#ifndef NANO_SMALL
-		do_mark
-#else
-		nano_disabled_msg
-#endif
-		);
-
     sc_init_one(&main_list, NANO_DELETE_KEY, N_("Delete"),
 	IFHELP(nano_delete_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, NOVIEW, do_delete);
@@ -531,9 +546,17 @@ void shortcut_init(int unjustify)
 	NANO_NO_KEY, VIEW, do_prev_word);
 #endif
 
-    sc_init_one(&main_list, NANO_NO_KEY, N_("Verbatim Input"),
-	IFHELP(nano_verbatim_msg, NANO_VERBATIM_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, NOVIEW, do_verbatim_input);
+#ifndef DISABLE_JUSTIFY
+    /* Translators: try to keep this string under 10 characters long */
+    sc_init_one(&main_list, NANO_NO_KEY, beg_of_par_msg,
+	IFHELP(nano_parabegin_msg, NANO_PARABEGIN_ALTKEY1), NANO_NO_KEY,
+	NANO_PARABEGIN_ALTKEY2, VIEW, do_para_begin);
+
+    /* Translators: try to keep this string under 10 characters long */
+    sc_init_one(&main_list, NANO_NO_KEY, end_of_par_msg,
+	IFHELP(nano_paraend_msg, NANO_PARAEND_ALTKEY1), NANO_NO_KEY,
+	NANO_PARAEND_ALTKEY2, VIEW, do_para_end);
+#endif
 
 #ifdef ENABLE_MULTIBUFFER
     sc_init_one(&main_list, NANO_NO_KEY, N_("Previous File"),
@@ -545,18 +568,23 @@ void shortcut_init(int unjustify)
 	NANO_OPENNEXT_ALTKEY, VIEW, open_nextfile_void);
 #endif
 
-#ifndef NANO_SMALL
-#ifdef HAVE_REGEX_H
+    sc_init_one(&main_list, NANO_NO_KEY, N_("Verbatim Input"),
+	IFHELP(nano_verbatim_msg, NANO_VERBATIM_KEY), NANO_NO_KEY,
+	NANO_NO_KEY, NOVIEW, do_verbatim_input);
+
+#ifndef DISABLE_JUSTIFY
+    /* Translators: try to keep this string under 10 characters long */
+    sc_init_one(&main_list, NANO_NO_KEY, fulljstify_msg,
+	IFHELP(nano_fulljustify_msg, NANO_FULLJUSTIFY_ALTKEY),
+	NANO_NO_KEY, NANO_NO_KEY, NOVIEW, do_full_justify);
+#endif
+
+#if !defined(NANO_SMALL) && defined(HAVE_REGEX_H)
     sc_init_one(&main_list, NANO_NO_KEY, N_("Find Other Bracket"),
 	IFHELP(nano_bracket_msg, NANO_BRACKET_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, do_find_bracket);
 #endif
 
-    sc_init_one(&main_list, NANO_NO_KEY, N_("Where Is Next"),
-	IFHELP(nano_whereis_next_msg, NANO_WHEREIS_NEXT_KEY),
-	NANO_NO_KEY, NANO_NO_KEY, VIEW, do_research);
-#endif
-
     free_shortcutage(&whereis_list);
 
     sc_init_one(&whereis_list, NANO_HELP_KEY, get_help_msg,
@@ -585,30 +613,25 @@ void shortcut_init(int unjustify)
 	NANO_NO_KEY, VIEW, do_last_line);
 
     /* Translators: try to keep this string under 10 characters long */
-    sc_init_one(&whereis_list, NANO_OTHERSEARCH_KEY, replace_msg,
+    sc_init_one(&whereis_list, NANO_TOOTHERSEARCH_KEY, replace_msg,
 	IFHELP(nano_replace_msg, NANO_NO_KEY), NANO_REPLACE_FKEY,
 	NANO_NO_KEY, VIEW, do_replace);
 
     /* Translators: try to keep this string under 10 characters long */
-    sc_init_one(&whereis_list, NANO_FROMSEARCHTOGOTO_KEY, go_to_line_msg,
-	IFHELP(nano_goto_msg, NANO_NO_KEY), NANO_GOTO_FKEY,
+    sc_init_one(&whereis_list, NANO_TOGOTOLINE_KEY, go_to_line_msg,
+	IFHELP(nano_gotoline_msg, NANO_NO_KEY), NANO_GOTOLINE_FKEY,
 	NANO_NO_KEY, VIEW, do_gotoline_void);
 
 #ifndef DISABLE_JUSTIFY
     /* Translators: try to keep this string under 10 characters long */
-    sc_init_one(&whereis_list, NANO_PARABEGIN_KEY, N_("Beg of Par"),
-	IFHELP(nano_parabegin_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_para_begin);
+    sc_init_one(&whereis_list, NANO_PARABEGIN_KEY, beg_of_par_msg,
+	IFHELP(nano_parabegin_msg, NANO_PARABEGIN_ALTKEY1), NANO_NO_KEY,
+	NANO_PARABEGIN_ALTKEY2, VIEW, do_para_begin);
 
     /* Translators: try to keep this string under 10 characters long */
-    sc_init_one(&whereis_list, NANO_PARAEND_KEY, N_("End of Par"),
-	IFHELP(nano_paraend_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_para_end);
-
-    /* Translators: try to keep this string under 10 characters long */
-    sc_init_one(&whereis_list, NANO_FULLJUSTIFY_KEY, N_("FullJstify"),
-	IFHELP(nano_fulljustify_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, NOVIEW, do_full_justify);
+    sc_init_one(&whereis_list, NANO_PARAEND_KEY, end_of_par_msg,
+	IFHELP(nano_paraend_msg, NANO_PARAEND_ALTKEY1), NANO_NO_KEY,
+	NANO_PARAEND_ALTKEY2, VIEW, do_para_end);
 #endif
 
 #ifndef NANO_SMALL
@@ -629,13 +652,17 @@ void shortcut_init(int unjustify)
 	NANO_NO_KEY, VIEW, 0);
 #endif
 
-#ifndef NANO_SMALL
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_HISTORY_KEY, history_msg,
 	IFHELP(nano_editstr_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, 0);
-#endif
 
+#ifndef DISABLE_JUSTIFY
+    /* Translators: try to keep this string under 10 characters long */
+    sc_init_one(&whereis_list, NANO_FULLJUSTIFY_KEY, fulljstify_msg,
+	IFHELP(nano_fulljustify_msg, NANO_FULLJUSTIFY_ALTKEY),
+	NANO_NO_KEY, NANO_NO_KEY, NOVIEW, do_full_justify);
+#endif
 #endif /* !NANO_SMALL */
 
     free_shortcutage(&replace_list);
@@ -663,12 +690,12 @@ void shortcut_init(int unjustify)
 	NANO_NO_KEY, VIEW, do_last_line);
 
     /* Translators: try to keep this string under 12 characters long */
-    sc_init_one(&replace_list, NANO_OTHERSEARCH_KEY, N_("No Replace"),
+    sc_init_one(&replace_list, NANO_TOOTHERSEARCH_KEY, N_("No Replace"),
 	IFHELP(nano_whereis_msg, NANO_NO_KEY), NANO_REPLACE_FKEY,
 	NANO_NO_KEY, VIEW, do_search);
 
-    sc_init_one(&replace_list, NANO_FROMSEARCHTOGOTO_KEY, go_to_line_msg,
-	IFHELP(nano_goto_msg, NANO_NO_KEY), NANO_GOTO_FKEY,
+    sc_init_one(&replace_list, NANO_TOGOTOLINE_KEY, go_to_line_msg,
+	IFHELP(nano_gotoline_msg, NANO_NO_KEY), NANO_GOTOLINE_FKEY,
 	NANO_NO_KEY, VIEW, do_gotoline_void);
 
 #ifndef NANO_SMALL
@@ -721,9 +748,9 @@ void shortcut_init(int unjustify)
 	NANO_NO_KEY, VIEW, 0);
 #endif
 
-    free_shortcutage(&goto_list);
+    free_shortcutage(&gotoline_list);
 
-    sc_init_one(&goto_list, NANO_HELP_KEY, get_help_msg,
+    sc_init_one(&gotoline_list, NANO_HELP_KEY, get_help_msg,
 	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
 	NANO_NO_KEY, VIEW,
 #ifndef DISABLE_HELP
@@ -733,18 +760,22 @@ void shortcut_init(int unjustify)
 #endif
 		);
 
-    sc_init_one(&goto_list, NANO_CANCEL_KEY, cancel_msg,
+    sc_init_one(&gotoline_list, NANO_CANCEL_KEY, cancel_msg,
 	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, 0);
 
-    sc_init_one(&goto_list, NANO_FIRSTLINE_KEY, first_line_msg,
+    sc_init_one(&gotoline_list, NANO_FIRSTLINE_KEY, first_line_msg,
 	IFHELP(nano_firstline_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, do_first_line);
 
-    sc_init_one(&goto_list, NANO_LASTLINE_KEY, last_line_msg,
+    sc_init_one(&gotoline_list, NANO_LASTLINE_KEY, last_line_msg,
 	IFHELP(nano_lastline_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, do_last_line);
 
+    sc_init_one(&gotoline_list, NANO_TOOTHERWHEREIS_KEY,
+	N_("Go To Text"), IFHELP(nano_whereis_msg, NANO_NO_KEY),
+	NANO_NO_KEY, NANO_NO_KEY, VIEW, do_search);
+
 #ifndef DISABLE_HELP
     free_shortcutage(&help_list);
 
@@ -865,18 +896,18 @@ void shortcut_init(int unjustify)
      * It's useless since inserting files is disabled. */
     /* Translators: try to keep this string under 22 characters long */
     if (!ISSET(RESTRICTED))
-	sc_init_one(&insertfile_list, NANO_EXTCMD_KEY, N_("Execute Command"),
-		IFHELP(nano_execute_msg, NANO_NO_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, NOVIEW, 0);
+	sc_init_one(&insertfile_list, NANO_TOOTHERINSERT_KEY,
+		N_("Execute Command"), IFHELP(nano_execute_msg,
+		NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY, NOVIEW, 0);
 
 #ifdef ENABLE_MULTIBUFFER
     /* If we're using restricted mode, the multibuffer toggle is
      * disabled.  It's useless since inserting files is disabled. */
     /* Translators: try to keep this string under 22 characters long */
     if (!ISSET(RESTRICTED))
-	sc_init_one(&insertfile_list, NANO_NO_KEY, _("New Buffer"),
-		IFHELP(nano_multibuffer_msg, TOGGLE_MULTIBUFFER_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, NOVIEW, 0);
+	sc_init_one(&insertfile_list, NANO_NO_KEY, new_buffer_msg,
+		IFHELP(nano_multibuffer_msg, TOGGLE_MULTIBUFFER_KEY),
+		NANO_NO_KEY, NANO_NO_KEY, NOVIEW, 0);
 #endif
 #endif
 
@@ -914,6 +945,16 @@ void shortcut_init(int unjustify)
     sc_init_one(&extcmd_list, NANO_CANCEL_KEY, cancel_msg,
 	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, VIEW, 0);
+
+    sc_init_one(&extcmd_list, NANO_TOOTHERINSERT_KEY, N_("Insert File"),
+	IFHELP(nano_insert_msg, NANO_NO_KEY), NANO_NO_KEY,
+	NANO_NO_KEY, VIEW, 0);
+
+#ifdef ENABLE_MULTIBUFFER
+    sc_init_one(&extcmd_list, NANO_NO_KEY, new_buffer_msg,
+	IFHELP(nano_multibuffer_msg, TOGGLE_MULTIBUFFER_KEY),
+	NANO_NO_KEY, NANO_NO_KEY, NOVIEW, 0);
+#endif
 #endif
 
 #ifndef DISABLE_BROWSER
@@ -942,9 +983,9 @@ void shortcut_init(int unjustify)
 	NANO_NO_KEY, VIEW, 0);
 
     /* Translators: try to keep this string under 22 characters long */
-    sc_init_one(&browser_list, NANO_GOTO_KEY, N_("Go To Dir"),
-	IFHELP(nano_gotodir_msg, NANO_ALT_GOTO_KEY), NANO_GOTO_FKEY,
-	NANO_NO_KEY, VIEW, 0);
+    sc_init_one(&browser_list, NANO_GOTOLINE_KEY, N_("Go To Dir"),
+	IFHELP(nano_gotodir_msg, NANO_GOTOLINE_ALTKEY),
+	NANO_GOTOLINE_FKEY, NANO_NO_KEY, VIEW, 0);
 
     free_shortcutage(&gotodir_list);
 
@@ -1034,17 +1075,13 @@ void toggle_init(void)
 #ifndef DISABLE_MOUSE
     toggle_init_one(TOGGLE_MOUSE_KEY, N_("Mouse support"), USE_MOUSE);
 #endif
-    /* If we're using restricted mode, the no-conversion, DOS format,
-     * Mac format, and backup toggles are disabled.  The first, second,
-     * and third are useless since inserting files is disabled, and the
-     * fourth is useless since backups are disabled. */
+    /* If we're using restricted mode, the no-conversion and backup
+     * backup toggles are disabled.  The former is useless since
+     * inserting files is disabled, and the latter is useless since
+     * backups are disabled. */
     if (!ISSET(RESTRICTED)) {
 	toggle_init_one(TOGGLE_NOCONVERT_KEY,
 		N_("No conversion from DOS/Mac format"), NO_CONVERT);
-	toggle_init_one(TOGGLE_DOS_KEY, N_("Writing file in DOS format"),
-		DOS_FILE);
-	toggle_init_one(TOGGLE_MAC_KEY, N_("Writing file in Mac format"),
-		MAC_FILE);
 	toggle_init_one(TOGGLE_BACKUP_KEY, N_("Backup files"), BACKUP_FILE);
     }
     toggle_init_one(TOGGLE_SMOOTH_KEY, N_("Smooth scrolling"), SMOOTHSCROLL);
@@ -1128,7 +1165,7 @@ void thanks_for_all_the_fish(void)
     free_shortcutage(&whereis_list);
     free_shortcutage(&replace_list);
     free_shortcutage(&replace_list_2);
-    free_shortcutage(&goto_list);
+    free_shortcutage(&gotoline_list);
     free_shortcutage(&writefile_list);
     free_shortcutage(&insertfile_list);
 #ifndef DISABLE_HELP
