@@ -1,4 +1,4 @@
-/* $Id: move.c,v 1.11 2004/07/28 20:46:25 dolorous Exp $ */
+/* $Id: move.c,v 1.15 2004/11/17 23:17:05 jordi Exp $ */
 /**************************************************************************
  *   move.c                                                               *
  *                                                                        *
@@ -19,7 +19,9 @@
  *                                                                        *
  **************************************************************************/
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <string.h>
@@ -35,7 +37,7 @@ void do_first_line(void)
     placewewant = 0;
     current_x = 0;
     if (edittop != fileage || need_vertical_update(old_pww))
-	edit_update(current, TOP);
+	edit_update(TOP);
 }
 
 void do_last_line(void)
@@ -46,7 +48,7 @@ void do_last_line(void)
     current_x = 0;
     if (edittop->lineno + (editwinrows / 2) != filebot->lineno ||
 	need_vertical_update(old_pww))
-	edit_update(current, CENTER);
+	edit_update(CENTER);
 }
 
 void do_home(void)
@@ -54,7 +56,7 @@ void do_home(void)
     size_t old_pww = placewewant;
 #ifndef NANO_SMALL
     if (ISSET(SMART_HOME)) {
-	int old_current_x = current_x;
+	size_t old_current_x = current_x;
 
 	for (current_x = 0; isblank(current->data[current_x]) &&
 		current->data[current_x] != '\0'; current_x++)
@@ -71,7 +73,7 @@ void do_home(void)
 #ifndef NANO_SMALL
     }
 #endif
-    check_statblank();
+    check_statusblank();
     if (need_horizontal_update(old_pww))
 	update_line(current, current_x);
 }
@@ -81,7 +83,7 @@ void do_end(void)
     size_t old_pww = placewewant;
     current_x = strlen(current->data);
     placewewant = xplustabs();
-    check_statblank();
+    check_statusblank();
     if (need_horizontal_update(old_pww))
 	update_line(current, current_x);
 }
@@ -129,7 +131,7 @@ void do_page_up(void)
     /* Update all the lines that need to be updated. */
     edit_redraw(old_current, old_pww);
 
-    check_statblank();
+    check_statusblank();
 }
 
 void do_page_down(void)
@@ -176,7 +178,7 @@ void do_page_down(void)
     /* Update all the lines that need to be updated. */
     edit_redraw(old_current, old_pww);
 
-    check_statblank();
+    check_statusblank();
 }
 
 void do_up(void)
@@ -184,7 +186,7 @@ void do_up(void)
 #ifndef DISABLE_WRAPPING
     wrap_reset();
 #endif
-    check_statblank();
+    check_statusblank();
 
     if (current->prev == NULL)
 	return;
@@ -217,7 +219,7 @@ void do_down(void)
 #ifndef DISABLE_WRAPPING
     wrap_reset();
 #endif
-    check_statblank();
+    check_statusblank();
 
     if (current->next == NULL)
 	return;
@@ -255,7 +257,7 @@ void do_left(int allow_update)
 	current_x = strlen(current->data);
     }
     placewewant = xplustabs();
-    check_statblank();
+    check_statusblank();
     if (allow_update && need_horizontal_update(old_pww))
 	update_line(current, current_x);
 }
@@ -277,7 +279,7 @@ void do_right(int allow_update)
 	current_x = 0;
     }
     placewewant = xplustabs();
-    check_statblank();
+    check_statusblank();
     if (allow_update && need_horizontal_update(old_pww))
 	update_line(current, current_x);
 }
