@@ -1,4 +1,4 @@
-/* $Id: nano.h,v 1.51 2004/08/17 05:23:38 dolorous Exp $ */
+/* $Id: nano.h,v 1.66 2004/11/07 23:05:48 dolorous Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -34,23 +34,25 @@
 #include <limits.h>
 #endif
 
-/* Macros for the flags long... */
+/* Macros for the flags long. */
 #define SET(bit) flags |= bit
 #define UNSET(bit) flags &= ~bit
 #define ISSET(bit) ((flags & bit) != 0)
 #define TOGGLE(bit) flags ^= bit
 
-/* Define charalloc as a macro rather than duplicating code */
+/* Macros for character allocation. */
 #define charalloc(howmuch) (char *)nmalloc((howmuch) * sizeof(char))
 #define charealloc(ptr, howmuch) (char *)nrealloc(ptr, (howmuch) * sizeof(char))
 #define charmove(dest, src, n) memmove(dest, src, (n) * sizeof(char))
+
 #ifdef BROKEN_REGEXEC
 #define regexec(preg, string, nmatch, pmatch, eflags) regexec_safe(preg, string, nmatch, pmatch, eflags)
 #endif
 
-#ifndef NANO_SMALL
-/* For the backup file copy ... */
-#define COPYFILEBLOCKSIZE 1024
+/* Set a default value for PATH_MAX, so we can use it in lines like
+ * "path = getcwd(NULL, PATH_MAX + 1);". */
+#ifndef PATH_MAX
+#define PATH_MAX -1
 #endif
 
 #ifdef USE_SLANG
@@ -142,6 +144,27 @@
 #define DISABLE_WRAPJUSTIFY 1
 #endif
 
+/* Enumeration types. */
+typedef enum {
+    CUT_LINE, CUT_MARKED, CUT_TO_EOL
+} cut_type;
+
+typedef enum {
+    NIX_FILE, DOS_FILE, MAC_FILE
+} file_format;
+
+typedef enum {
+    UP, DOWN
+} updown;
+
+typedef enum {
+    TOP, CENTER, NONE
+} topmidnone;
+
+typedef enum {
+    NO_SEQ, ESCAPE_SEQ, UTF8_SEQ
+} seq_type;
+
 /* Structure types. */
 typedef struct filestruct {
     char *data;
@@ -164,24 +187,34 @@ typedef struct openfilestruct {
     struct filestruct *file_mark_beginbuf;
 				/* Current file's beginning marked
 				 * line. */
-    int file_mark_beginx;	/* Current file's beginning marked
+    size_t file_mark_beginx;	/* Current file's beginning marked
 				 * line's x-coordinate position. */
 #endif
-    int file_current_x;		/* Current file's x-coordinate
+    size_t file_current_x;	/* Current file's x-coordinate
 				 * position. */
     int file_current_y;		/* Current file's y-coordinate
 				 * position. */
-    long file_flags;		/* Current file's flags: modification
-				 * status (and marking status, if
-				 * available). */
     size_t file_placewewant;	/* Current file's place we want. */
     int file_totlines;		/* Current file's total number of
 				 * lines. */
     long file_totsize;		/* Current file's total size. */
     int file_lineno;		/* Current file's line number. */
+    long file_flags;		/* Current file's flags: modification
+				 * status (and marking status, if
+				 * available). */
+    file_format file_fmt;	/* Current file's format. */
 } openfilestruct;
 #endif
 
+typedef struct partition {
+    filestruct *fileage;
+    filestruct *top_prev;
+    char *top_data;
+    filestruct *filebot;
+    filestruct *bot_next;
+    char *bot_data;
+} partition;
+
 typedef struct shortcut {
     /* Key values that aren't used should be set to NANO_NO_KEY. */
     int ctrlval;	/* Special sentinel key or control key we want
@@ -208,17 +241,16 @@ typedef struct toggle {
    long flag;		/* What flag actually gets toggled. */
    struct toggle *next;
 } toggle;
-#endif /* !NANO_SMALL */
+#endif
 
 #ifdef ENABLE_NANORC
 typedef struct rcoption {
    const char *name;
    long flag;
 } rcoption;
-#endif /* ENABLE_NANORC */
+#endif
 
 #ifdef ENABLE_COLOR
-
 typedef struct colortype {
     int fg;			/* fg color */
     int bg;			/* bg color */
@@ -240,8 +272,7 @@ typedef struct syntaxtype {
     colortype *color;		/* color struct for this syntax */
     struct syntaxtype *next;
 } syntaxtype;
-
-#endif /* ENABLE_COLOR */
+#endif
 
 #ifndef NANO_SMALL
 typedef struct historytype {
@@ -249,6 +280,7 @@ typedef struct historytype {
     struct historytype *prev;
     char *data;
 } historytype;
+
 typedef struct historyheadtype {
     struct historytype *next;	/* Keep *next and *prev members
 				 * together. */
@@ -259,7 +291,7 @@ typedef struct historyheadtype {
     int count;
     int len;
 } historyheadtype;
-#endif /* !NANO_SMALL */
+#endif
 
 /* Bitwise flags so we can save space (or more correctly, not waste
  * it). */
@@ -279,21 +311,19 @@ typedef struct historyheadtype {
 #define CUT_TO_END		(1<<13)
 #define REVERSE_SEARCH		(1<<14)
 #define MULTIBUFFER		(1<<15)
-#define DOS_FILE		(1<<16)
-#define MAC_FILE		(1<<17)
-#define SMOOTHSCROLL		(1<<18)
-#define DISABLE_CURPOS		(1<<19)	/* Damn, we still need it. */
-#define REBIND_DELETE		(1<<20)
-#define NO_CONVERT		(1<<21)
-#define BACKUP_FILE		(1<<22)
-#define NO_RCFILE		(1<<23)
-#define COLOR_SYNTAX		(1<<24)
-#define PRESERVE		(1<<25)
-#define HISTORY_CHANGED		(1<<26)
-#define HISTORYLOG		(1<<27)
-#define RESTRICTED		(1<<28)
-#define SMART_HOME		(1<<29)
-#define WHITESPACE_DISPLAY	(1<<30)
+#define SMOOTHSCROLL		(1<<16)
+#define DISABLE_CURPOS		(1<<17)	/* Damn, we still need it. */
+#define REBIND_DELETE		(1<<18)
+#define NO_CONVERT		(1<<19)
+#define BACKUP_FILE		(1<<20)
+#define NO_RCFILE		(1<<21)
+#define COLOR_SYNTAX		(1<<22)
+#define PRESERVE		(1<<23)
+#define HISTORY_CHANGED		(1<<24)
+#define HISTORYLOG		(1<<25)
+#define RESTRICTED		(1<<26)
+#define SMART_HOME		(1<<27)
+#define WHITESPACE_DISPLAY	(1<<28)
 
 /* Control key sequences, changing these would be very very bad. */
 #define NANO_CONTROL_SPACE 0
@@ -330,6 +360,8 @@ typedef struct historyheadtype {
 #define NANO_CONTROL_7 31
 #define NANO_CONTROL_8 127
 
+#define NANO_ALT_9 '9'
+#define NANO_ALT_0 '0'
 #define NANO_ALT_A 'a'
 #define NANO_ALT_B 'b'
 #define NANO_ALT_C 'c'
@@ -358,6 +390,8 @@ typedef struct historyheadtype {
 #define NANO_ALT_Z 'z'
 #define NANO_ALT_PERIOD '.'
 #define NANO_ALT_COMMA ','
+#define NANO_ALT_LPAREN '('
+#define NANO_ALT_RPAREN ')'
 #define NANO_ALT_LCARAT '<'
 #define NANO_ALT_RCARAT '>'
 #define NANO_ALT_RBRACKET ']'
@@ -375,24 +409,29 @@ typedef struct historyheadtype {
 /* Normal keys. */
 #define NANO_XON_KEY		NANO_CONTROL_Q
 #define NANO_XOFF_KEY		NANO_CONTROL_S
-#define NANO_INSERTFILE_KEY	NANO_CONTROL_R
-#define NANO_INSERTFILE_FKEY	KEY_F(5)
+#define NANO_CANCEL_KEY		NANO_CONTROL_C
 #define NANO_EXIT_KEY		NANO_CONTROL_X
 #define NANO_EXIT_FKEY		KEY_F(2)
+#define NANO_INSERTFILE_KEY	NANO_CONTROL_R
+#define NANO_INSERTFILE_FKEY	KEY_F(5)
+#define NANO_TOOTHERINSERT_KEY	NANO_CONTROL_X
 #define NANO_WRITEOUT_KEY	NANO_CONTROL_O
 #define NANO_WRITEOUT_FKEY	KEY_F(3)
-#define NANO_GOTO_KEY		NANO_CONTROL_7
-#define NANO_GOTO_FKEY		KEY_F(13)
-#define NANO_ALT_GOTO_KEY	NANO_ALT_G
+#define NANO_GOTOLINE_KEY	NANO_CONTROL_7
+#define NANO_GOTOLINE_FKEY	KEY_F(13)
+#define NANO_GOTOLINE_ALTKEY	NANO_ALT_G
+#define NANO_TOGOTOLINE_KEY	NANO_CONTROL_T
 #define NANO_HELP_KEY		NANO_CONTROL_G
 #define NANO_HELP_FKEY		KEY_F(1)
 #define NANO_WHEREIS_KEY	NANO_CONTROL_W
 #define NANO_WHEREIS_FKEY	KEY_F(6)
 #define NANO_WHEREIS_NEXT_KEY	NANO_ALT_W
+#define NANO_WHEREIS_NEXT_FKEY	KEY_F(16)
+#define NANO_TOOTHERWHEREIS_KEY	NANO_CONTROL_T
 #define NANO_REPLACE_KEY	NANO_CONTROL_4
 #define NANO_REPLACE_FKEY	KEY_F(14)
 #define NANO_ALT_REPLACE_KEY	NANO_ALT_R
-#define NANO_OTHERSEARCH_KEY	NANO_CONTROL_R
+#define NANO_TOOTHERSEARCH_KEY	NANO_CONTROL_R
 #define NANO_PREVPAGE_KEY	NANO_CONTROL_Y
 #define NANO_PREVPAGE_FKEY	KEY_F(7)
 #define NANO_NEXTPAGE_KEY	NANO_CONTROL_V
@@ -409,7 +448,6 @@ typedef struct historyheadtype {
 #define NANO_FIRSTLINE_FKEY	NANO_PREVPAGE_FKEY
 #define NANO_LASTLINE_KEY	NANO_NEXTPAGE_KEY
 #define NANO_LASTLINE_FKEY	NANO_NEXTPAGE_FKEY
-#define NANO_CANCEL_KEY		NANO_CONTROL_C
 #define NANO_REFRESH_KEY	NANO_CONTROL_L
 #define NANO_JUSTIFY_KEY	NANO_CONTROL_J
 #define NANO_JUSTIFY_FKEY	KEY_F(4)
@@ -420,7 +458,8 @@ typedef struct historyheadtype {
 #define NANO_FORWARD_KEY	NANO_CONTROL_F
 #define NANO_BACK_KEY		NANO_CONTROL_B
 #define NANO_MARK_KEY		NANO_CONTROL_6
-#define NANO_ALT_MARK_KEY	NANO_ALT_A
+#define NANO_MARK_ALTKEY	NANO_ALT_A
+#define NANO_MARK_FKEY		KEY_F(15)
 #define NANO_HOME_KEY		NANO_CONTROL_A
 #define NANO_END_KEY		NANO_CONTROL_E
 #define NANO_DELETE_KEY		NANO_CONTROL_D
@@ -428,7 +467,6 @@ typedef struct historyheadtype {
 #define NANO_TAB_KEY		NANO_CONTROL_I
 #define NANO_SUSPEND_KEY	NANO_CONTROL_Z
 #define NANO_ENTER_KEY		NANO_CONTROL_M
-#define NANO_FROMSEARCHTOGOTO_KEY NANO_CONTROL_T
 #define NANO_TOFILES_KEY	NANO_CONTROL_T
 #define NANO_APPEND_KEY		NANO_ALT_A
 #define NANO_PREPEND_KEY	NANO_ALT_P
@@ -437,12 +475,16 @@ typedef struct historyheadtype {
 #define NANO_OPENPREV_ALTKEY	NANO_ALT_COMMA
 #define NANO_OPENNEXT_ALTKEY	NANO_ALT_PERIOD
 #define NANO_BRACKET_KEY	NANO_ALT_RBRACKET
-#define NANO_EXTCMD_KEY		NANO_CONTROL_X
 #define NANO_NEXTWORD_KEY	NANO_CONTROL_SPACE
 #define NANO_PREVWORD_KEY	NANO_ALT_SPACE
 #define NANO_PARABEGIN_KEY	NANO_CONTROL_W
+#define NANO_PARABEGIN_ALTKEY1	NANO_ALT_LPAREN
+#define NANO_PARABEGIN_ALTKEY2	NANO_ALT_9
 #define NANO_PARAEND_KEY	NANO_CONTROL_O
+#define NANO_PARAEND_ALTKEY1	NANO_ALT_RPAREN
+#define NANO_PARAEND_ALTKEY2	NANO_ALT_0
 #define NANO_FULLJUSTIFY_KEY	NANO_CONTROL_U
+#define NANO_FULLJUSTIFY_ALTKEY	NANO_ALT_J
 #define NANO_VERBATIM_KEY	NANO_ALT_V
 
 #ifndef NANO_SMALL
@@ -459,7 +501,7 @@ typedef struct historyheadtype {
 #define TOGGLE_CASE_KEY		NANO_ALT_C
 #define TOGGLE_MULTIBUFFER_KEY	NANO_ALT_F
 #define TOGGLE_DOS_KEY		NANO_ALT_D
-#define TOGGLE_MAC_KEY		NANO_ALT_O
+#define TOGGLE_MAC_KEY		NANO_ALT_M
 #define TOGGLE_SMOOTH_KEY	NANO_ALT_S
 #define TOGGLE_NOCONVERT_KEY	NANO_ALT_N
 #define TOGGLE_BACKUP_KEY	NANO_ALT_B
@@ -473,14 +515,6 @@ typedef struct historyheadtype {
 #define VIEW TRUE
 #define NOVIEW FALSE
 
-typedef enum {
-    UP, DOWN
-} updown;
-
-typedef enum {
-    TOP, CENTER, NONE
-} topmidnone;
-
 /* Minimum editor window rows required for nano to work correctly. */
 #define MIN_EDITOR_ROWS 3
 
@@ -491,6 +525,9 @@ typedef enum {
  * occurs. */
 #define CHARS_FROM_EOL 8
 
+/* Default width of a tab. */
+#define WIDTH_OF_TAB 8
+
 /* Maximum number of search history strings saved, same value used for
  * replace history. */
 #define MAX_SEARCH_HISTORY 100
