@@ -1,4 +1,4 @@
-/* $Id: proto.h,v 1.81 2004/08/17 20:34:57 dolorous Exp $ */
+/* $Id: proto.h,v 1.123 2004/11/22 00:16:23 dolorous Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -21,8 +21,8 @@
 
 /* Externs. */
 
-#include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #ifdef HAVE_REGEX_H
 #include <regex.h>
 #endif
@@ -32,15 +32,16 @@
 extern ssize_t wrap_at;
 #endif
 extern int editwinrows;
-extern int current_x, current_y, totlines;
+extern size_t current_x;
+extern int current_y;
+extern int totlines;
 extern size_t placewewant;
 #ifndef NANO_SMALL
-extern int mark_beginx;
+extern size_t mark_beginx;
 #endif
 extern long totsize;
 extern long flags;
 extern ssize_t tabsize;
-extern int search_last_line;
 extern int currslen;
 
 #if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
@@ -82,10 +83,10 @@ extern char *full_operating_dir;
 extern char *alt_speller;
 #endif
 
-extern int resetstatuspos;
 extern struct stat fileinfo;
 extern filestruct *current, *fileage, *edittop, *filebot;
 extern filestruct *cutbuffer;
+extern partition *filepart;
 #ifndef NANO_SMALL
 extern filestruct *mark_beginbuf;
 #endif
@@ -102,7 +103,7 @@ extern char *syntaxstr;
 
 extern shortcut *shortcut_list;
 extern shortcut *main_list, *whereis_list;
-extern shortcut *replace_list, *goto_list;
+extern shortcut *replace_list, *gotoline_list;
 extern shortcut *writefile_list, *insertfile_list;
 extern shortcut *replace_list_2;
 #ifndef NANO_SMALL
@@ -148,30 +149,42 @@ extern char *homedir;
 
 /* Functions we want available. */
 
-/* Public functions in color.c */
+/* Public functions in color.c. */
 #ifdef ENABLE_COLOR
 void set_colorpairs(void);
 void do_colorinit(void);
 void update_color(void);
 #endif /* ENABLE_COLOR */
 
-/* Public functions in cut.c */
+/* Public functions in cut.c. */
 void cutbuffer_reset(void);
 filestruct *get_cutbottom(void);
 void add_to_cutbuffer(filestruct *inptr, bool allow_concat);
+#ifndef NANO_SMALL
 void cut_marked_segment(void);
+#endif
 void do_cut_text(void);
 void do_uncut_text(void);
 
-/* Public functions in files.c */
-void load_file(int update);
+/* Public functions in files.c. */
 void new_file(void);
-filestruct *read_line(char *buf, filestruct *prev, int *line1ins, size_t
-	len);
-void read_file(FILE *f, const char *filename, int quiet);
-bool open_file(const char *filename, int insert, int quiet);
+filestruct *read_line(char *buf, filestruct *prev, bool *first_line_ins,
+	size_t len);
+void load_file(void);
+void read_file(FILE *f, const char *filename);
+int open_file(const char *filename, bool newfie, FILE **f);
 char *get_next_filename(const char *name);
-void do_insertfile(int loading_file);
+#ifndef NANO_SMALL
+void execute_command(const char *command);
+#endif
+void load_buffer(const char *name);
+void do_insertfile(
+#ifndef NANO_SMALL
+	bool execute
+#else
+	void
+#endif
+	);
 void do_insertfile_void(void);
 #ifdef ENABLE_MULTIBUFFER
 openfilestruct *make_new_opennode(openfilestruct *prevnode);
@@ -180,13 +193,13 @@ void splice_opennode(openfilestruct *begin, openfilestruct *newnode,
 void unlink_opennode(const openfilestruct *fileptr);
 void delete_opennode(openfilestruct *fileptr);
 void free_openfilestruct(openfilestruct *src);
-void add_open_file(int update);
+void add_open_file(bool update);
 void load_open_file(void);
-void open_prevfile(int closing_file);
+void open_prevfile(bool closing_file);
 void open_prevfile_void(void);
-void open_nextfile(int closing_file);
+void open_nextfile(bool closing_file);
 void open_nextfile_void(void);
-int close_open_file(void);
+bool close_open_file(void);
 #endif
 #if !defined(DISABLE_SPELLER) || !defined(DISABLE_OPERATINGDIR)
 char *get_full_path(const char *origpath);
@@ -203,11 +216,12 @@ int check_operating_dir(const char *currpath, int allow_tabcomp);
 void init_backup_dir(void);
 #endif
 int copy_file(FILE *inn, FILE *out);
-int write_file(const char *name, int tmp, int append, int nonamechange);
+int write_file(const char *name, bool tmp, int append, bool
+	nonamechange);
 #ifndef NANO_SMALL
-int write_marked(const char *name, int tmp, int append);
+int write_marked(const char *name, bool tmp, int append);
 #endif
-int do_writeout(int exiting);
+int do_writeout(bool exiting);
 void do_writeout_void(void);
 char *real_dir_from_tilde(const char *buf);
 #ifndef DISABLE_TABCOMP
@@ -234,7 +248,7 @@ bool writehist(FILE *hist, historyheadtype *histhead);
 void save_history(void);
 #endif
 
-/* Public functions in global.c */
+/* Public functions in global.c. */
 size_t length_of_list(const shortcut *s);
 #ifndef NANO_SMALL
 void toggle_init_one(int val, const char *desc, long flag);
@@ -249,13 +263,13 @@ void sc_init_one(shortcut **shortcutage, int key, const char *desc,
 #endif
 	int metaval, int funcval, int miscval, int view, void
 	(*func)(void));
-void shortcut_init(int unjustify);
+void shortcut_init(bool unjustify);
 void free_shortcutage(shortcut **shortcutage);
 #ifdef DEBUG
 void thanks_for_all_the_fish(void);
 #endif
 
-/* Public functions in move.c */
+/* Public functions in move.c. */
 void do_first_line(void);
 void do_last_line(void);
 void do_home(void);
@@ -269,7 +283,7 @@ void do_left_void(void);
 void do_right(int allow_update);
 void do_right_void(void);
 
-/* Public functions in nano.c */
+/* Public functions in nano.c. */
 void finish(void);
 void die(const char *msg, ...);
 void die_save_file(const char *die_filename);
@@ -291,6 +305,9 @@ void unlink_node(const filestruct *fileptr);
 void delete_node(filestruct *fileptr);
 filestruct *copy_filestruct(const filestruct *src);
 void free_filestruct(filestruct *src);
+partition *partition_filestruct(filestruct *top, size_t top_x,
+	filestruct *bot, size_t bot_x);
+void unpartition_filestruct(partition **p);
 void renumber_all(void);
 void renumber(filestruct *fileptr);
 void print1opt(const char *shortflag, const char *longflag, const char
@@ -303,9 +320,6 @@ void nano_disabled_msg(void);
 RETSIGTYPE cancel_fork(int signal);
 bool open_pipe(const char *command);
 #endif
-#ifndef DISABLE_MOUSE
-void do_mouse(void);
-#endif
 void do_char(char ch);
 void do_verbatim_input(void);
 void do_backspace(void);
@@ -343,8 +357,8 @@ bool inpar(const char *str);
 void do_para_end(void);
 filestruct *backup_lines(filestruct *first_line, size_t par_len, size_t
 	quote_len);
-bool breakable(const char *line, int goal);
-int break_line(const char *line, int goal, bool force);
+bool breakable(const char *line, ssize_t goal);
+ssize_t break_line(const char *line, ssize_t goal, bool force);
 bool do_para_search(size_t *const quote, size_t *const par);
 void do_justify(bool full_justify);
 void do_justify_void(void);
@@ -362,6 +376,7 @@ void allow_pending_sigwinch(bool allow);
 #ifndef NANO_SMALL
 void do_toggle(const toggle *which);
 #endif
+void disable_extended_input(void);
 void disable_signals(void);
 #ifndef NANO_SMALL
 void enable_signals(void);
@@ -370,7 +385,7 @@ void disable_flow_control(void);
 void enable_flow_control(void);
 void terminal_init(void);
 
-/* Public functions in rcfile.c */
+/* Public functions in rcfile.c. */
 #ifdef ENABLE_NANORC
 void rcfile_error(const char *msg, ...);
 char *parse_next_word(char *ptr);
@@ -386,7 +401,7 @@ void parse_rcfile(FILE *rcstream);
 void do_rcfile(void);
 #endif /* ENABLE_NANORC */
 
-/* Public functions in search.c */
+/* Public functions in search.c. */
 #ifdef HAVE_REGEX_H
 int regexp_init(const char *regexp);
 void regexp_cleanup(void);
@@ -394,27 +409,30 @@ void regexp_cleanup(void);
 void not_found_msg(const char *str);
 void search_abort(void);
 void search_init_globals(void);
-int search_init(int replacing);
-int is_whole_word(int curr_pos, const char *datastr, const char
+int search_init(bool replacing, bool use_answer);
+bool is_whole_word(int curr_pos, const char *datastr, const char
 	*searchword);
-int findnextstr(int can_display_wrap, int wholeword, const filestruct
-	*begin, size_t beginx, const char *needle, int no_sameline);
+bool findnextstr(bool can_display_wrap, bool wholeword, bool
+	no_sameline, const filestruct *begin, size_t beginx, const char
+	*needle, size_t *needle_len);
+void findnextstr_wrap_reset(void);
 void do_search(void);
 #ifndef NANO_SMALL
 void do_research(void);
 #endif
 void replace_abort(void);
 #ifdef HAVE_REGEX_H
-int replace_regexp(char *string, int create_flag);
+int replace_regexp(char *string, bool create_flag);
 #endif
 char *replace_line(const char *needle);
-int do_replace_loop(const char *needle, const filestruct *real_current,
-	size_t *real_current_x, int wholewords);
+ssize_t do_replace_loop(const char *needle, const filestruct
+	*real_current, size_t *real_current_x, bool wholewords, bool
+	*canceled);
 void do_replace(void);
 void do_gotoline(int line, bool save_pos);
 void do_gotoline_void(void);
-#if defined (ENABLE_MULTIBUFFER) || !defined (DISABLE_SPELLER)
-void do_gotopos(int line, int pos_x, int pos_y, size_t pos_pww);
+#if defined(ENABLE_MULTIBUFFER) || !defined(DISABLE_SPELLER)
+void do_gotopos(int line, size_t pos_x, int pos_y, size_t pos_pww);
 #endif
 void do_find_bracket(void);
 #ifndef NANO_SMALL
@@ -431,7 +449,7 @@ void free_history(historyheadtype *h);
 #endif
 #endif
 
-/* Public functions in utils.c */
+/* Public functions in utils.c. */
 #ifdef HAVE_REGEX_H
 #ifdef BROKEN_REGEXEC
 int regexec_safe(const regex_t *preg, const char *string, size_t nmatch,
@@ -480,45 +498,60 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 void nperror(const char *s);
 void *nmalloc(size_t howmuch);
 void *nrealloc(void *ptr, size_t howmuch);
+char *mallocstrncpy(char *dest, const char *src, size_t n);
 char *mallocstrcpy(char *dest, const char *src);
+char *mallocstrassn(char *dest, char *src);
 void new_magicline(void);
 #ifndef NANO_SMALL
+void remove_magicline(void);
+void get_totals(const filestruct *begin, const filestruct *end, int
+	*lines, long *size);
 void mark_order(const filestruct **top, size_t *top_x, const filestruct
-	**bot, size_t *bot_x);
+	**bot, size_t *bot_x, bool *right_side_up);
 #endif
 #ifndef DISABLE_TABCOMP
 int check_wildcard_match(const char *text, const char *pattern);
 #endif
 
-/* Public functions in winio.c */
+/* Public functions in winio.c. */
 #ifndef NANO_SMALL
 void reset_kbinput(void);
 #endif
-int get_kbinput(WINDOW *win, int *meta_key);
-int get_translated_kbinput(int kbinput, int *es
+void unget_kbinput(int kbinput, bool meta_key, bool func_key);
+int get_kbinput(WINDOW *win, bool *meta_key, bool *func_key);
+int get_translated_kbinput(int kbinput, seq_type *seq
 #ifndef NANO_SMALL
-	, int reset
+	, bool reset
 #endif
 	);
 int get_ascii_kbinput(int kbinput, size_t ascii_digits
 #ifndef NANO_SMALL
-	, int reset
+	, bool reset
 #endif
 	);
 int get_control_kbinput(int kbinput);
-int get_escape_seq_kbinput(int *escape_seq, size_t es_len, int
+int get_escape_seq_kbinput(const int *escape_seq, size_t es_len, bool
 	*ignore_seq);
 int get_escape_seq_abcd(int kbinput);
-int *get_verbatim_kbinput(WINDOW *win, int *v_kbinput, size_t *v_len,
-	int allow_ascii);
-int get_untranslated_kbinput(int kbinput, size_t position, int
+int *get_verbatim_kbinput(WINDOW *win, int first, int *v_kbinput, size_t
+	*v_len, bool allow_ascii);
+int get_untranslated_kbinput(int kbinput, size_t position, bool
 	allow_ascii
 #ifndef NANO_SMALL
-	, int reset
+	, bool reset
 #endif
 	);
 #ifndef DISABLE_MOUSE
-int get_mouseinput(int *mouse_x, int *mouse_y, int allow_shortcuts);
+bool get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts);
+#endif
+const shortcut *get_shortcut(const shortcut *s_list, int kbinput, bool
+	*meta_key, bool *func_key);
+#ifndef NANO_SMALL
+const toggle *get_toggle(int kbinput, bool meta_key);
+#endif
+int get_edit_input(bool *meta_key, bool *func_key, bool allow_funcs);
+#ifndef DISABLE_MOUSE
+bool get_edit_mouse(void);
 #endif
 size_t xplustabs(void);
 size_t actual_x(const char *str, size_t xplus);
@@ -527,11 +560,11 @@ size_t strlenpt(const char *buf);
 void blank_titlebar(void);
 void blank_edit(void);
 void blank_statusbar(void);
-void check_statblank(void);
+void check_statusblank(void);
 void blank_bottombars(void);
 char *display_string(const char *buf, size_t start_col, size_t len);
 void nanoget_repaint(const char *buf, const char *inputbuf, size_t x);
-int nanogetstr(int allowtabs, const char *buf, const char *def,
+int nanogetstr(bool allow_tabs, const char *buf, const char *def,
 #ifndef NANO_SMALL
 		historyheadtype *history_list,
 #endif
@@ -540,6 +573,12 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 		, bool *list
 #endif
 		);
+int statusq(bool allow_tabs, const shortcut *s, const char *def,
+#ifndef NANO_SMALL
+		historyheadtype *history_list,
+#endif
+		const char *msg, ...);
+void statusq_abort(void);
 void titlebar(const char *path);
 void set_modified(void);
 void statusbar(const char *msg, ...);
@@ -558,16 +597,11 @@ int need_vertical_update(size_t old_pww);
 void edit_scroll(updown direction, int nlines);
 void edit_redraw(const filestruct *old_current, size_t old_pww);
 void edit_refresh(void);
-void edit_update(filestruct *fileptr, topmidnone location);
-int statusq(int allowtabs, const shortcut *s, const char *def,
-#ifndef NANO_SMALL
-		historyheadtype *history_list,
-#endif
-		const char *msg, ...);
-int do_yesno(int all, const char *msg);
+void edit_update(topmidnone location);
+int do_yesno(bool all, const char *msg);
 void total_refresh(void);
 void display_main_list(void);
-void do_cursorpos(int constant);
+void do_cursorpos(bool constant);
 void do_cursorpos_void(void);
 #ifndef DISABLE_HELP
 int help_line_len(const char *ptr);
