@@ -1,4 +1,4 @@
-/* $Id: rcfile.c,v 1.32 2004/08/17 05:23:38 dolorous Exp $ */
+/* $Id: rcfile.c,v 1.36 2004/11/17 23:17:05 jordi Exp $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -19,7 +19,9 @@
  *                                                                        *
  **************************************************************************/
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <stdarg.h>
@@ -27,7 +29,6 @@
 #include <stdio.h>
 #include <errno.h>
 #include <unistd.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 #include <pwd.h>
 #include <ctype.h>
@@ -611,20 +612,21 @@ void parse_rcfile(FILE *rcstream)
 			    else
 #endif
 			    if (strcasecmp(rcopts[i].name, "tabsize") == 0) {
-				if (!parse_num(option, &tabsize) || tabsize <= 0)
+				if (!parse_num(option, &tabsize) || tabsize <= 0) {
 				    rcfile_error(N_("Requested tab size %s invalid"), option);
 				    tabsize = -1;
+				}
 			    }
 			} else
 			    SET(rcopts[i].flag);
 #ifdef DEBUG
-			fprintf(stderr, "set flag %d!\n",
+			fprintf(stderr, "set flag %ld!\n",
 				rcopts[i].flag);
 #endif
 		    } else {
 			UNSET(rcopts[i].flag);
 #ifdef DEBUG
-			fprintf(stderr, "unset flag %d!\n",
+			fprintf(stderr, "unset flag %ld!\n",
 				rcopts[i].flag);
 #endif
 		    }
