@@ -1,4 +1,4 @@
-/* $Id: search.c,v 1.35 2004/08/11 05:13:08 dolorous Exp $ */
+/* $Id: search.c,v 1.80 2004/11/22 17:08:41 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -19,7 +19,9 @@
  *                                                                        *
  **************************************************************************/
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <string.h>
@@ -31,9 +33,11 @@
 #include "proto.h"
 #include "nano.h"
 
-
+static bool search_last_line = FALSE;
+	/* Have we gone past the last line while searching? */
 #ifdef HAVE_REGEX_H
-static int regexp_compiled = FALSE;
+static bool regexp_compiled = FALSE;
+	/* Have we compiled any regular expressions? */
 
 /* Regular expression helper functions. */
 
@@ -42,8 +46,11 @@ static int regexp_compiled = FALSE;
  * bar.  Return value 1 means success. */
 int regexp_init(const char *regexp)
 {
-    int rc = regcomp(&search_regexp, regexp, REG_EXTENDED |
-	(ISSET(CASE_SENSITIVE) ? 0 : REG_ICASE));
+    int rc = regcomp(&search_regexp, regexp, REG_EXTENDED
+#ifndef NANO_SMALL
+	| (ISSET(CASE_SENSITIVE) ? 0 : REG_ICASE)
+#endif
+	);
 
     assert(!regexp_compiled);
     if (rc != 0) {
@@ -99,42 +106,45 @@ void search_abort(void)
 
 void search_init_globals(void)
 {
-    if (last_search == NULL) {
-	last_search = charalloc(1);
-	last_search[0] = '\0';
-    }
-    if (last_replace == NULL) {
-	last_replace = charalloc(1);
-	last_replace[0] = '\0';
-    }
+    if (last_search == NULL)
+	last_search = mallocstrcpy(NULL, "");
+    if (last_replace == NULL)
+	last_replace = mallocstrcpy(NULL, "");
 }
 
-/* Set up the system variables for a search or replace.  Return -1 if
- * the search should be cancelled (due to Cancel, Go to Line, or a
- * failed regcomp()).  Return 0 on success, and 1 on rerun calling
- * program.  Return -2 to run opposite program (search -> replace,
- * replace -> search).
+/* Set up the system variables for a search or replace.  If use_answer
+ * is TRUE, only set backupstring to answer.  Return -2 to run opposite
+ * program (search -> replace, replace -> search), return -1 if the
+ * search should be canceled (due to Cancel, Go to Line, or a failed
+ * regcomp()), return 0 on success, and return 1 on rerun calling
+ * program.
  *
- * replacing = 1 if we call from do_replace(), 0 if called from
- * do_search(). */
-int search_init(int replacing)
+ * replacing is TRUE if we call from do_replace(), and FALSE if called
+ * from do_search(). */
+int search_init(bool replacing, bool use_answer)
 {
     int i = 0;
     char *buf;
     static char *backupstring = NULL;
+	/* The search string we'll be using. */
+
+    /* If backupstring doesn't exist, initialize it to "". */
+    if (backupstring == NULL)
+	backupstring = mallocstrcpy(NULL, "");
+
+    /* If use_answer is TRUE, set backupstring to answer and get out. */
+    if (use_answer) {
+	backupstring = mallocstrcpy(backupstring, answer);
+	return 0;
+    }
 
     /* We display the search prompt below.  If the user types a partial
      * search string and then Replace or a toggle, we will return to
      * do_search() or do_replace() and be called again.  In that case,
-     * we should put the same search string back up.  backupstring holds
-     * this string. */
+     * we should put the same search string back up. */
 
     search_init_globals();
 
-    /* If we don't already have a backupstring, set it. */
-    if (backupstring == NULL)
-	backupstring = mallocstrcpy(NULL, "");
-
 #ifndef NANO_SMALL
     search_history.current = (historytype *)&search_history.next;
 #endif
@@ -148,10 +158,8 @@ int search_init(int replacing)
 	sprintf(buf, " [%s%s]", disp,
 		strlenpt(last_search) > COLS / 3 ? "..." : "");
 	free(disp);
-    } else {
-	buf = charalloc(1);
-	buf[0] = '\0';
-    }
+    } else
+	buf = mallocstrcpy(NULL, "");
 
     /* This is now one simple call.  It just does a lot. */
     i = statusq(FALSE, replacing ? replace_list : whereis_list,
@@ -182,7 +190,16 @@ int search_init(int replacing)
 #endif
 		"",
 
-	replacing ? _(" (to replace)") : "",
+	replacing ?
+#ifndef NANO_SMALL
+		(ISSET(MARK_ISSET) ? _(" (to replace) in selection") :
+#endif
+		_(" (to replace)")
+#ifndef NANO_SMALL
+		)
+#endif
+		: "",
+
 	buf);
 
     /* Release buf now that we don't need it anymore. */
@@ -194,129 +211,174 @@ int search_init(int replacing)
     /* Cancel any search, or just return with no previous search. */
     if (i == -1 || (i < 0 && last_search[0] == '\0') ||
 	    (!replacing && i == 0 && answer[0] == '\0')) {
-	statusbar(_("Search Cancelled"));
+	statusbar(_("Cancelled"));
 #ifndef NANO_SMALL
 	search_history.current = search_history.next;
 #endif
 	return -1;
     } else {
 	switch (i) {
-	case -2:	/* It's the same string. */
+	    case -2:		/* It's the same string. */
 #ifdef HAVE_REGEX_H
-	    /* Since answer is "", use last_search! */
-	    if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0)
-		return -1;
+		/* Since answer is "", use last_search! */
+		if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0)
+		    return -1;
 #endif
-	    break;
-	case 0:		/* They entered something new. */
-	    last_replace[0] = '\0';
+		break;
+	    case 0:		/* They entered something new. */
+		last_replace[0] = '\0';
 #ifdef HAVE_REGEX_H
-	    if (ISSET(USE_REGEXP) && regexp_init(answer) == 0)
-		return -1;
+		if (ISSET(USE_REGEXP) && regexp_init(answer) == 0)
+		    return -1;
 #endif
-	    break;
+		break;
 #ifndef NANO_SMALL
-	case TOGGLE_CASE_KEY:
-	    TOGGLE(CASE_SENSITIVE);
-	    backupstring = mallocstrcpy(backupstring, answer);
-	    return 1;
-	case TOGGLE_BACKWARDS_KEY:
-	    TOGGLE(REVERSE_SEARCH);
-	    backupstring = mallocstrcpy(backupstring, answer);
-	    return 1;
+	    case TOGGLE_CASE_KEY:
+		TOGGLE(CASE_SENSITIVE);
+		backupstring = mallocstrcpy(backupstring, answer);
+		return 1;
+	    case TOGGLE_BACKWARDS_KEY:
+		TOGGLE(REVERSE_SEARCH);
+		backupstring = mallocstrcpy(backupstring, answer);
+		return 1;
 #ifdef HAVE_REGEX_H
-	case TOGGLE_REGEXP_KEY:
-	    TOGGLE(USE_REGEXP);
-	    backupstring = mallocstrcpy(backupstring, answer);
-	    return 1;
+	    case TOGGLE_REGEXP_KEY:
+		TOGGLE(USE_REGEXP);
+		backupstring = mallocstrcpy(backupstring, answer);
+		return 1;
 #endif
 #endif /* !NANO_SMALL */
-	case NANO_OTHERSEARCH_KEY:
-	    backupstring = mallocstrcpy(backupstring, answer);
-	    return -2;	/* Call the opposite search function. */
-	case NANO_FROMSEARCHTOGOTO_KEY:
+	    case NANO_TOOTHERSEARCH_KEY:
+		backupstring = mallocstrcpy(backupstring, answer);
+		return -2;	/* Call the opposite search function. */
+	    case NANO_TOGOTOLINE_KEY:
 #ifndef NANO_SMALL
-	    search_history.current = search_history.next;
+		search_history.current = search_history.next;
 #endif
-	   /* If answer parses as an integer, put it up on the
-	    * statusbar. */
-	    do_gotoline(parse_num(answer, NULL) ? -1 : 0, FALSE);
-	    /* Fall through. */
-	default:
-	    return -1;
+		/* Put answer up on the statusbar. */
+		do_gotoline(-1, FALSE);
+		/* Fall through. */
+	    default:
+		return -1;
 	}
     }
     return 0;
 }
 
-int is_whole_word(int curr_pos, const char *datastr, const char
+bool is_whole_word(int curr_pos, const char *datastr, const char
 	*searchword)
 {
     size_t sln = curr_pos + strlen(searchword);
 
     /* Start of line or previous character is not a letter and end of
      * line or next character is not a letter. */
-    return (curr_pos < 1 || !isalpha((int)datastr[curr_pos - 1])) &&
-	(sln == strlen(datastr) || !isalpha((int)datastr[sln]));
+    return (curr_pos < 1 || !isalpha(datastr[curr_pos - 1])) &&
+	(sln == strlen(datastr) || !isalpha(datastr[sln]));
 }
 
 /* Look for needle, starting at current, column current_x.  If
- * no_sameline is nonzero, skip over begin when looking for needle.
- * begin is the line where we first started searching, at column beginx.
- * If can_display_wrap is nonzero, we put messages on the statusbar, and
- * wrap around the file boundaries.  The return value specifies whether
- * we found anything. */
-int findnextstr(int can_display_wrap, int wholeword, const filestruct
-	*begin, size_t beginx, const char *needle, int no_sameline)
+ * no_sameline is TRUE, skip over begin when looking for needle.  begin
+ * is the line where we first started searching, at column beginx.  If
+ * can_display_wrap is TRUE, we put messages on the statusbar, wrap
+ * around the file boundaries.  The return value specifies whether we
+ * found anything.  If we did, set needle_len to the length of the
+ * string we found if it isn't NULL. */
+bool findnextstr(bool can_display_wrap, bool wholeword, bool
+	no_sameline, const filestruct *begin, size_t beginx, const char
+	*needle, size_t *needle_len)
 {
     filestruct *fileptr = current;
     const char *rev_start = NULL, *found = NULL;
+    size_t found_len;
+	/* The length of the match we found. */
     size_t current_x_find = 0;
-	/* Where needle was found. */
+	/* The location of the match we found. */
+    int current_y_find = current_y;
 
     /* rev_start might end up 1 character before the start or after the
      * end of the line.  This won't be a problem because strstrwrapper()
      * will return immediately and say that no match was found, and
      * rev_start will be properly set when the search continues on the
      * previous or next line. */
+    rev_start =
 #ifndef NANO_SMALL
-    if (ISSET(REVERSE_SEARCH))
-	rev_start = fileptr->data + (current_x - 1);
-    else
+	ISSET(REVERSE_SEARCH) ? fileptr->data + (current_x - 1) :
 #endif
-	rev_start = fileptr->data + (current_x + 1);
+	fileptr->data + (current_x + 1);
 
     /* Look for needle in searchstr. */
     while (TRUE) {
 	found = strstrwrapper(fileptr->data, needle, rev_start);
 
-	if (found != NULL && (!wholeword || is_whole_word(found -
-		fileptr->data, fileptr->data, needle))) {
-	    if (!no_sameline || fileptr != current)
+	/* We've found a potential match. */
+	if (found != NULL) {
+	    bool found_whole = FALSE;
+		/* Is this potential match a whole word? */
+
+	    /* Set found_len to the length of the potential match. */
+	    found_len =
+#ifdef HAVE_REGEX_H
+		ISSET(USE_REGEXP) ?
+		regmatches[0].rm_eo - regmatches[0].rm_so :
+#endif
+		strlen(needle);
+
+	    /* If we're searching for whole words, see if this potential
+	     * match is a whole word. */
+	    if (wholeword) {
+		char *word = mallocstrncpy(NULL, found, found_len + 1);
+		word[found_len] = '\0';
+
+		found_whole = is_whole_word(found - fileptr->data,
+			fileptr->data, word);
+		free(word);
+	    }
+
+	    /* If we're searching for whole words and this potential
+	     * match isn't a whole word, or if we're not allowed to find
+	     * a match on the same line we started on and this potential
+	     * match is on that line, continue searching. */
+	    if ((!wholeword || found_whole) && (!no_sameline ||
+		fileptr != current))
 		break;
 	}
 
-	/* Finished processing file, get out. */
+	/* We've finished processing the file, so get out. */
 	if (search_last_line) {
 	    if (can_display_wrap)
 		not_found_msg(needle);
-	    return 0;
+	    return FALSE;
 	}
-	fileptr =
-#ifndef NANO_SMALL
-		ISSET(REVERSE_SEARCH) ? fileptr->prev :
-#endif
-		fileptr->next;
 
-	/* Start or end of buffer reached; wrap around. */
+#ifndef NANO_SMALL
+	if (ISSET(REVERSE_SEARCH)) {
+	    fileptr = fileptr->prev;
+	    current_y_find--;
+	} else {
+#endif
+	    fileptr = fileptr->next;
+	    current_y_find++;
+#ifndef NANO_SMALL
+	}
+#endif
+
+	/* Start or end of buffer reached, so wrap around. */
 	if (fileptr == NULL) {
 	    if (!can_display_wrap)
-		return 0;
-	    fileptr =
+		return FALSE;
+
 #ifndef NANO_SMALL
-		ISSET(REVERSE_SEARCH) ? filebot :
+	    if (ISSET(REVERSE_SEARCH)) {
+		fileptr = filebot;
+		current_y_find = editwinrows - 1;
+	    } else {
 #endif
-		fileage;
+		fileptr = fileage;
+		current_y_find = 0;
+#ifndef NANO_SMALL
+	    }
+#endif
+
 	    if (can_display_wrap)
 		statusbar(_("Search Wrapped"));
 	}
@@ -324,6 +386,7 @@ int findnextstr(int can_display_wrap, int wholeword, const filestruct
 	/* Original start line reached. */
 	if (fileptr == begin)
 	    search_last_line = TRUE;
+
 	rev_start = fileptr->data;
 #ifndef NANO_SMALL
 	if (ISSET(REVERSE_SEARCH))
@@ -346,27 +409,40 @@ int findnextstr(int can_display_wrap, int wholeword, const filestruct
 
 	if (can_display_wrap)
 	    not_found_msg(needle);
-	return 0;
+	return FALSE;
     }
 
     /* Set globals now that we are sure we found something. */
     current = fileptr;
     current_x = current_x_find;
+    current_y = current_y_find;
+    placewewant = xplustabs();
 
-    return 1;
+    /* needle_len holds the length of needle. */
+    if (needle_len != NULL)
+	*needle_len = found_len;
+
+    return TRUE;
+}
+
+void findnextstr_wrap_reset(void)
+{
+    search_last_line = FALSE;
 }
 
 /* Search for a string. */
 void do_search(void)
 {
     size_t old_pww = placewewant, fileptr_x = current_x;
-    int i, didfind;
+    int i;
+    bool didfind;
     filestruct *fileptr = current;
 
 #ifndef DISABLE_WRAPPING
     wrap_reset();
 #endif
-    i = search_init(0);
+
+    i = search_init(FALSE, FALSE);
     if (i == -1)	/* Cancel, Go to Line, blank search string, or
 			 * regcomp() failed. */
 	search_abort();
@@ -394,8 +470,9 @@ void do_search(void)
 	update_history(&search_history, answer);
 #endif
 
-    search_last_line = FALSE;
-    didfind = findnextstr(TRUE, FALSE, current, current_x, answer, FALSE);
+    findnextstr_wrap_reset();
+    didfind = findnextstr(TRUE, FALSE, FALSE, current, current_x,
+	answer, NULL);
 
     /* Check to see if there's only one occurrence of the string and
      * we're on it now. */
@@ -408,8 +485,8 @@ void do_search(void)
 	 * which case it's the only occurrence. */
 	if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
 		last_search)) {
-	    didfind = findnextstr(TRUE, FALSE, current, current_x,
-		answer, TRUE);
+	    didfind = findnextstr(TRUE, FALSE, TRUE, current, current_x,
+		answer, NULL);
 	    if (fileptr == current && fileptr_x == current_x && !didfind)
 		statusbar(_("This is the only occurrence"));
 	} else {
@@ -430,12 +507,13 @@ void do_search(void)
 void do_research(void)
 {
     size_t old_pww = placewewant, fileptr_x = current_x;
-    int didfind;
+    bool didfind;
     filestruct *fileptr = current;
 
 #ifndef DISABLE_WRAPPING
     wrap_reset();
 #endif
+
     search_init_globals();
 
     if (last_search[0] != '\0') {
@@ -446,9 +524,9 @@ void do_research(void)
 	    return;
 #endif
 
-	search_last_line = FALSE;
-	didfind = findnextstr(TRUE, FALSE, current, current_x,
-		last_search, FALSE);
+	findnextstr_wrap_reset();
+	didfind = findnextstr(TRUE, FALSE, FALSE, current, current_x,
+		last_search, NULL);
 
 	/* Check to see if there's only one occurrence of the string and
 	 * we're on it now. */
@@ -461,8 +539,8 @@ void do_research(void)
 	     * found again, in which case it's the only occurrence. */
 	    if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
 		last_search)) {
-		didfind = findnextstr(TRUE, FALSE, current, current_x,
-			answer, TRUE);
+		didfind = findnextstr(TRUE, FALSE, TRUE, current,
+			current_x, answer, NULL);
 		if (fileptr == current && fileptr_x == current_x && !didfind)
 		    statusbar(_("This is the only occurrence"));
 	    } else {
@@ -491,9 +569,9 @@ void replace_abort(void)
 }
 
 #ifdef HAVE_REGEX_H
-int replace_regexp(char *string, int create_flag)
+int replace_regexp(char *string, bool create_flag)
 {
-    /* Split personality here - if create_flag is zero, just calculate
+    /* Split personality here - if create_flag is FALSE, just calculate
      * the size of the replacement line (necessary because of
      * subexpressions \1 to \9 in the replaced text). */
 
@@ -506,7 +584,8 @@ int replace_regexp(char *string, int create_flag)
     while (*c != '\0') {
 	int num = (int)(*(c + 1) - '0');
 
-	if (*c != '\\' || num < 1 || num > 9 || num > search_regexp.re_nsub) {
+	if (*c != '\\' || num < 1 || num > 9 || num >
+		search_regexp.re_nsub) {
 	    if (create_flag)
 		*string++ = *c;
 	    c++;
@@ -520,7 +599,7 @@ int replace_regexp(char *string, int create_flag)
 	    /* But add the length of the subexpression to new_size. */
 	    new_size += i;
 
-	    /* And if create_flag is nonzero, append the result of the
+	    /* And if create_flag is TRUE, append the result of the
 	     * subexpression match to the new line. */
 	    if (create_flag) {
 		strncpy(string, current->data + current_x +
@@ -579,31 +658,51 @@ char *replace_line(const char *needle)
 }
 
 /* Step through each replace word and prompt user before replacing.
- * Parameters real_current and real_current_x are needed by the internal
- * speller, to allow the cursor position to be updated when a word
- * before the cursor is replaced by a shorter word.
+ * Parameters real_current and real_current_x are needed in order to
+ * allow the cursor position to be updated when a word before the cursor
+ * is replaced by a shorter word.
  *
  * needle is the string to seek.  We replace it with answer.  Return -1
- * if needle isn't found, else the number of replacements performed. */
-int do_replace_loop(const char *needle, const filestruct *real_current,
-	size_t *real_current_x, int wholewords)
+ * if needle isn't found, else the number of replacements performed.  If
+ * canceled isn't NULL, set it to TRUE if we canceled. */
+ssize_t do_replace_loop(const char *needle, const filestruct
+	*real_current, size_t *real_current_x, bool wholewords, bool
+	*canceled)
 {
-    int replaceall = FALSE, numreplaced = -1;
-    size_t old_pww = placewewant, current_x_save = current_x;
-    const filestruct *current_save = current;
+    ssize_t numreplaced = -1;
+    size_t match_len;
+    size_t pww_save = placewewant;
+    bool replaceall = FALSE;
 #ifdef HAVE_REGEX_H
     /* The starting-line match and bol/eol regex flags. */
-    int begin_line = FALSE, bol_or_eol = FALSE;
+    bool begin_line = FALSE, bol_or_eol = FALSE;
 #endif
 #ifndef NANO_SMALL
-    int mark_set = ISSET(MARK_ISSET);
+    bool old_mark_set = ISSET(MARK_ISSET);
+    filestruct *edittop_save = edittop, *top, *bot;
+    size_t top_x, bot_x;
+    bool right_side_up = FALSE;
+	/* TRUE if (mark_beginbuf, mark_beginx) is the top of the mark,
+	 * FALSE if (current, current_x) is. */
 
-    UNSET(MARK_ISSET);
-    edit_refresh();
+    if (old_mark_set) {
+	/* If the mark is on, partition the filestruct so that it
+	 * contains only the marked text, set edittop to the top of the
+	 * partition, turn the mark off, and refresh the screen. */
+	mark_order((const filestruct **)&top, &top_x,
+	    (const filestruct **)&bot, &bot_x, &right_side_up);
+	filepart = partition_filestruct(top, top_x, bot, bot_x);
+	edittop = fileage;
+	UNSET(MARK_ISSET);
+	edit_refresh();
+    }
 #endif
 
-    while (findnextstr(TRUE, wholewords, current_save, current_x_save,
-	needle,
+    if (canceled != NULL)
+	*canceled = FALSE;
+
+    findnextstr_wrap_reset();
+    while (findnextstr(TRUE, wholewords,
 #ifdef HAVE_REGEX_H
 	/* We should find a bol and/or eol regex only once per line.  If
 	 * the bol_or_eol flag is set, it means that the last search
@@ -613,10 +712,9 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 #else
 	FALSE
 #endif
-	) != 0) {
+	, real_current, *real_current_x, needle, &match_len)) {
 
 	int i = 0;
-	size_t match_len;
 
 #ifdef HAVE_REGEX_H
 	/* If the bol_or_eol flag is set, we've found a match on the
@@ -636,19 +734,12 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 #endif
 
 	if (!replaceall) {
-	    placewewant = xplustabs();
-	    edit_redraw(current_save, old_pww);
-	    old_pww = placewewant;
+	    edit_redraw(real_current, pww_save);
+	    pww_save = placewewant;
 	}
 
-#ifdef HAVE_REGEX_H
-	if (ISSET(USE_REGEXP))
-	    match_len = regmatches[0].rm_eo - regmatches[0].rm_so;
-	else
-#endif
-	    match_len = strlen(needle);
-
-	/* Record for the return value that we found the search string. */
+	/* Record for the return value that we found the search
+	 * string. */
 	if (numreplaced == -1)
 	    numreplaced = 0;
 
@@ -668,8 +759,11 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 	    free(exp_word);
 	    curs_set(1);
 
-	    if (i == -1)	/* We canceled the replace. */
+	    if (i == -1) {	/* We canceled the replace. */
+		if (canceled != NULL)
+		    *canceled = TRUE;
 		break;
+	    }
 	}
 
 #ifdef HAVE_REGEX_H
@@ -682,7 +776,7 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 
 	if (i > 0 || replaceall) {	/* Yes, replace it!!!! */
 	    char *copy;
-	    int length_change;
+	    size_t length_change;
 
 	    if (i == 2)
 		replaceall = TRUE;
@@ -692,24 +786,38 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 	    length_change = strlen(copy) - strlen(current->data);
 
 #ifndef NANO_SMALL
-	    if (current == mark_beginbuf && mark_beginx > current_x) {
-		if (mark_beginx < current_x + match_len)
-		    mark_beginx = current_x;
-		else
-		    mark_beginx += length_change;
+	    /* If the mark was on and (mark_beginbuf, mark_begin_x) was
+	     * the top of it, don't change mark_beginx. */
+	    if (!old_mark_set || !right_side_up) {
+		/* Keep mark_beginx in sync with the text changes. */
+		if (current == mark_beginbuf && mark_beginx >
+			current_x) {
+		    if (mark_beginx < current_x + match_len)
+			mark_beginx = current_x;
+		    else
+			mark_beginx += length_change;
+		}
+	    }
+
+	    /* If the mark was on and (current, current_x) was the top
+	     * of it, don't change real_current_x. */
+	    if (!old_mark_set || right_side_up) {
+#endif
+		/* Keep real_current_x in sync with the text changes. */
+		if (current == real_current && current_x <=
+			*real_current_x) {
+		    if (*real_current_x < current_x + match_len)
+			*real_current_x = current_x + match_len;
+		    *real_current_x += length_change;
+		}
+#ifndef NANO_SMALL
 	    }
 #endif
 
-	    assert(0 <= match_len + length_change);
-	    if (current == real_current && current_x <= *real_current_x) {
-		if (*real_current_x < current_x + match_len)
-		    *real_current_x = current_x + match_len;
-		*real_current_x += length_change;
-	    }
-
 	    /* Set the cursor at the last character of the replacement
 	     * text, so searching will resume after the replacement
-	     * text.  Note that current_x might be set to -1 here. */
+	     * text.  Note that current_x might be set to (size_t)-1
+	     * here. */
 #ifndef NANO_SMALL
 	    if (!ISSET(REVERSE_SEARCH))
 #endif
@@ -734,24 +842,32 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 	}
     }
 
+#ifndef NANO_SMALL
+    if (old_mark_set) {
+	/* If the mark was on, unpartition the filestruct so that it
+	 * contains all the text again, set edittop back to what it was
+	 * before, turn the mark back on, and refresh the screen. */
+	unpartition_filestruct(&filepart);
+	edittop = edittop_save;
+	SET(MARK_ISSET);
+	edit_refresh();
+    }
+#endif
+
     /* If text has been added to the magicline, make a new magicline. */
     if (filebot->data[0] != '\0')
 	new_magicline();
 
-#ifndef NANO_SMALL
-    if (mark_set)
-	SET(MARK_ISSET);
-#endif
-
     return numreplaced;
 }
 
 /* Replace a string. */
 void do_replace(void)
 {
-    int i, numreplaced;
+    int i;
     filestruct *edittop_save, *begin;
-    size_t beginx;
+    size_t beginx, pww_save;
+    ssize_t numreplaced;
 
     if (ISSET(VIEW_MODE)) {
 	print_view_warning();
@@ -759,7 +875,7 @@ void do_replace(void)
 	return;
     }
 
-    i = search_init(1);
+    i = search_init(TRUE, FALSE);
     if (i == -1) {		/* Cancel, Go to Line, blank search
 				 * string, or regcomp() failed. */
 	replace_abort();
@@ -805,7 +921,7 @@ void do_replace(void)
 	if (i == -1) {		/* Cancel. */
 	    if (last_replace[0] != '\0')
 		answer = mallocstrcpy(answer, last_replace);
-	    statusbar(_("Replace Cancelled"));
+	    statusbar(_("Cancelled"));
 	}
 	replace_abort();
 	return;
@@ -814,23 +930,26 @@ void do_replace(void)
     last_replace = mallocstrcpy(last_replace, answer);
 
     /* Save where we are. */
+    edittop_save = edittop;
     begin = current;
     beginx = current_x;
-    edittop_save = edittop;
-    search_last_line = FALSE;
+    pww_save = placewewant;
 
-    numreplaced = do_replace_loop(last_search, begin, &beginx, FALSE);
+    numreplaced = do_replace_loop(last_search, begin, &beginx, FALSE,
+	NULL);
 
     /* Restore where we were. */
     edittop = edittop_save;
     current = begin;
     current_x = beginx;
+    placewewant = pww_save;
+
     renumber_all();
     edit_refresh();
 
     if (numreplaced >= 0)
-	statusbar(P_("Replaced %d occurrence", "Replaced %d occurrences",
-		numreplaced), numreplaced);
+	statusbar(P_("Replaced %ld occurrence", "Replaced %ld occurrences",
+		(long)numreplaced), (long)numreplaced);
 
     replace_abort();
 }
@@ -839,7 +958,7 @@ void do_gotoline(int line, bool save_pos)
 {
     if (line <= 0) {		/* Ask for it. */
 	char *ans = mallocstrcpy(NULL, answer);
-	int st = statusq(FALSE, goto_list, line < 0 ? ans : "",
+	int i = statusq(FALSE, gotoline_list, line < 0 ? ans : "",
 #ifndef NANO_SMALL
 		NULL,
 #endif
@@ -848,16 +967,25 @@ void do_gotoline(int line, bool save_pos)
 	free(ans);
 
 	/* Cancel, or Enter with blank string. */
-	if (st == -1 || st == -2)
-	    statusbar(_("Aborted"));
-	if (st != 0) {
+	if (i < 0) {
+	    statusbar(_("Cancelled"));
 	    display_main_list();
 	    return;
 	}
 
-	/* Bounds check. */
+	if (i == NANO_TOOTHERWHEREIS_KEY) {
+	    /* Keep answer up on the statusbar. */
+	    search_init(TRUE, TRUE);
+
+	    do_search();
+	    return;
+	}
+
+	/* Do a bounds check.  Display a warning on an out-of-bounds
+	 * line number only if we hit Enter at the statusbar prompt. */
 	if (!parse_num(answer, &line) || line < 0) {
-	    statusbar(_("Come on, be reasonable"));
+	    if (i == 0)
+		statusbar(_("Come on, be reasonable"));
 	    display_main_list();
 	    return;
 	}
@@ -877,7 +1005,7 @@ void do_gotoline(int line, bool save_pos)
 
     /* If save_pos is TRUE, don't change the cursor position when
      * updating the edit window. */
-    edit_update(current, save_pos ? NONE : CENTER);
+    edit_update(save_pos ? NONE : CENTER);
 
     placewewant = 0;
     display_main_list();
@@ -889,7 +1017,7 @@ void do_gotoline_void(void)
 }
 
 #if defined(ENABLE_MULTIBUFFER) || !defined(DISABLE_SPELLER)
-void do_gotopos(int line, int pos_x, int pos_y, size_t pos_pww)
+void do_gotopos(int line, size_t pos_x, int pos_y, size_t pos_pww)
 {
     /* Since do_gotoline() resets the x-coordinate but not the
      * y-coordinate, set the coordinates up this way. */
@@ -913,7 +1041,7 @@ void do_find_bracket(void)
     char ch_under_cursor, wanted_ch;
     const char *pos, *brackets = "([{<>}])";
     char regexp_pat[] = "[  ]";
-    size_t old_pww, current_x_save;
+    size_t current_x_save, pww_save;
     int count = 1;
     long flags_save;
     filestruct *current_save;
@@ -929,9 +1057,9 @@ void do_find_bracket(void)
     assert(strlen(brackets) % 2 == 0);
     wanted_ch = brackets[(strlen(brackets) - 1) - (pos - brackets)];
 
-    old_pww = placewewant;
-    current_x_save = current_x;
     current_save = current;
+    current_x_save = current_x;
+    pww_save = placewewant;
     flags_save = flags;
     SET(USE_REGEXP);
 
@@ -954,24 +1082,24 @@ void do_find_bracket(void)
     /* We constructed regexp_pat to be a valid expression. */
     assert(regexp_compiled);
 
-    search_last_line = FALSE;
+    findnextstr_wrap_reset();
     while (TRUE) {
-	if (findnextstr(FALSE, FALSE, current, current_x, regexp_pat,
-		FALSE) != 0) {
+	if (findnextstr(FALSE, FALSE, FALSE, current, current_x,
+		regexp_pat, NULL)) {
 	    /* Found identical bracket. */
 	    if (current->data[current_x] == ch_under_cursor)
 		count++;
 	    /* Found complementary bracket. */
 	    else if (--count == 0) {
 		placewewant = xplustabs();
-		edit_redraw(current_save, old_pww);
+		edit_redraw(current_save, pww_save);
 		break;
 	    }
 	} else {
 	    /* Didn't find either a left or right bracket. */
 	    statusbar(_("No matching bracket"));
-	    current_x = current_x_save;
 	    current = current_save;
+	    current_x = current_x_save;
 	    update_line(current, current_x);
 	    break;
 	}
