@@ -1,4 +1,4 @@
-/* $Id: utils.c,v 1.23 2004/08/17 20:34:57 dolorous Exp $ */
+/* $Id: utils.c,v 1.40 2004/11/22 16:04:18 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
@@ -19,7 +19,9 @@
  *                                                                        *
  **************************************************************************/
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <string.h>
@@ -44,11 +46,10 @@ int regexec_safe(const regex_t *preg, const char *string, size_t nmatch,
 #define regexec(preg, string, nmatch, pmatch, eflags) regexec_safe(preg, string, nmatch, pmatch, eflags)
 #endif /* BROKEN_REGEXEC */
 
-/* Assume that string will be found by regexec() if the REG_NOTBOL and
- * REG_NOTEOL glags are not set. */
 int regexp_bol_or_eol(const regex_t *preg, const char *string)
 {
-    return (regexec(preg, string, 0, NULL, REG_NOTBOL | REG_NOTEOL) ==
+    return (regexec(preg, string, 0, NULL, 0) == 0 &&
+	regexec(preg, string, 0, NULL, REG_NOTBOL | REG_NOTEOL) ==
 	REG_NOMATCH);
 }
 #endif /* HAVE_REGEX_H */
@@ -84,9 +85,9 @@ int num_of_digits(int n)
     return i;
 }
 
-/* Read an int from str, and store it in *val (if val is not NULL).  On
- * error, we return FALSE and don't change *val.  Otherwise, we return 
- * TRUE. */
+/* Read a ssize_t from str, and store it in *val (if val is not NULL).
+ * On error, we return FALSE and don't change *val.  Otherwise, we
+ * return TRUE. */
 bool parse_num(const char *str, ssize_t *val)
 {
     char *first_error;
@@ -310,7 +311,7 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 	const char *start)
 {
     /* start can be 1 character before the start or after the end of the
-     * line.  In either case, we just say there is no match found. */
+     * line.  In either case, we just say no match was found. */
     if ((start > haystack && *(start - 1) == '\0') || start < haystack)
 	return NULL;
     assert(haystack != NULL && needle != NULL && start != NULL);
@@ -322,7 +323,7 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 		&& haystack + regmatches[0].rm_so <= start) {
 		const char *retval = haystack + regmatches[0].rm_so;
 
-		/* Search forward until there is no more match. */
+		/* Search forward until there are no more matches. */
 		while (regexec(&search_regexp, retval + 1, 1, regmatches,
 			REG_NOTBOL) == 0 && retval + 1 +
 			regmatches[0].rm_so <= start)
@@ -395,9 +396,10 @@ void *nrealloc(void *ptr, size_t howmuch)
     return r;
 }
 
-/* Copy one malloc()ed string to another pointer.  Should be used as:
- * dest = mallocstrcpy(dest, src); */
-char *mallocstrcpy(char *dest, const char *src)
+/* Copy the first n characters of one malloc()ed string to another
+ * pointer.  Should be used as: "dest = mallocstrncpy(dest, src,
+ * n);". */
+char *mallocstrncpy(char *dest, const char *src, size_t n)
 {
     if (src == NULL)
 	src = "";
@@ -405,18 +407,33 @@ char *mallocstrcpy(char *dest, const char *src)
     if (src != dest)
 	free(dest);
 
-    dest = charalloc(strlen(src) + 1);
-    strcpy(dest, src);
+    dest = charalloc(n);
+    strncpy(dest, src, n);
 
     return dest;
 }
 
-/* Append a new magic-line to filebot. */
+/* Copy one malloc()ed string to another pointer.  Should be used as:
+ * "dest = mallocstrcpy(dest, src);". */
+char *mallocstrcpy(char *dest, const char *src)
+{
+    return mallocstrncpy(dest, src, src == NULL ? 1 : strlen(src) + 1);
+}
+
+/* Free the malloc()ed string at dest and return the malloc()ed string
+ * at src.  Should be used as: "answer = mallocstrassn(answer,
+ * real_dir_from_tilde(answer));". */
+char *mallocstrassn(char *dest, char *src)
+{
+    free(dest);
+    return src;
+}
+
+/* Append a new magicline to filebot. */
 void new_magicline(void)
 {
     filebot->next = (filestruct *)nmalloc(sizeof(filestruct));
-    filebot->next->data = charalloc(1);
-    filebot->next->data[0] = '\0';
+    filebot->next->data = mallocstrcpy(NULL, "");
     filebot->next->prev = filebot;
     filebot->next->next = NULL;
     filebot->next->lineno = filebot->lineno + 1;
@@ -426,23 +443,89 @@ void new_magicline(void)
 }
 
 #ifndef NANO_SMALL
+/* Remove the magicline from filebot, if there is one and it isn't the
+ * only line in the file. */
+void remove_magicline(void)
+{
+    if (filebot->data[0] == '\0' && filebot->prev != NULL) {
+	filebot = filebot->prev;
+	free_filestruct(filebot->next);
+	filebot->next = NULL;
+	totlines--;
+	totsize--;
+    }
+}
+
+/* Calculate the number of lines and the number of characters between
+ * begin and end, and return them in lines and size, respectively. */
+void get_totals(const filestruct *begin, const filestruct *end, int
+	*lines, long *size)
+{
+    const filestruct *f;
+
+    if (lines != NULL)
+	*lines = 0;
+    if (size != NULL)
+	*size = 0;
+
+    /* Go through the lines from begin to end->prev, if we can. */
+    for (f = begin; f != NULL && f != end; f = f->next) {
+	/* Count this line. */
+	if (lines != NULL)
+	    (*lines)++;
+
+	/* Count the number of characters on this line. */
+	if (size != NULL) {
+	    *size += strlen(f->data);
+
+	    /* Count the newline if we have one. */
+	    if (f->next != NULL)
+		(*size)++;
+	}
+    }
+
+    /* Go through the line at end, if we can. */
+    if (f != NULL) {
+	/* Count this line. */
+	if (lines != NULL)
+	    (*lines)++;
+
+	/* Count the number of characters on this line. */
+	if (size != NULL) {
+	    *size += strlen(f->data);
+
+	    /* Count the newline if we have one. */
+	    if (f->next != NULL)
+		(*size)++;
+	}
+    }
+}
+
 /* Set top_x and bot_x to the top and bottom x-coordinates of the mark,
- * respectively, based on the locations of top and bot. */
+ * respectively, based on the locations of top and bot.  If
+ * right_side_up isn't NULL, set it to TRUE If the mark begins with
+ * (mark_beginbuf, mark_beginx) and ends with (current, current_x), or
+ * FALSE otherwise. */
 void mark_order(const filestruct **top, size_t *top_x, const filestruct
-	**bot, size_t *bot_x)
+	**bot, size_t *bot_x, bool *right_side_up)
 {
     assert(top != NULL && top_x != NULL && bot != NULL && bot_x != NULL);
-    if ((current->lineno == mark_beginbuf->lineno && current_x > mark_beginx)
-	|| current->lineno > mark_beginbuf->lineno) {
+
+    if ((current->lineno == mark_beginbuf->lineno && current_x >
+	 mark_beginx) || current->lineno > mark_beginbuf->lineno) {
 	*top = mark_beginbuf;
 	*top_x = mark_beginx;
 	*bot = current;
 	*bot_x = current_x;
+	if (right_side_up != NULL)
+	    *right_side_up = TRUE;
     } else {
 	*bot = mark_beginbuf;
 	*bot_x = mark_beginx;
 	*top = current;
 	*top_x = current_x;
+	if (right_side_up != NULL)
+	    *right_side_up = FALSE;
     }
 }
 #endif
